import 'package:flutter/material.dart';
import 'package:oc_front/core/sections/end_drawer.dart';
import 'package:oc_front/core/services/specialized_services/shared_service.dart';
import 'package:oc_front/models/shared.dart';
import 'package:oc_front/models/workspace.dart';
import 'package:oc_front/pages/catalog.dart';
import 'package:oc_front/pages/catalog_item.dart';
import 'package:oc_front/pages/workflow.dart';

class WorkSpaceItem {
  String? id;
  String? name;
  WorkSpaceItem({this.id, this.name});
}

class CollaborativeAreaLocal {
  static String? current;
  static Map<String, CollaborativeArea> workspaces = {};
  static final SharedService _service = SharedService();

  static Future<void> init(BuildContext context, bool changeCurrent) async {
      await _service.all(context).then((value) {
      if (value.data != null && value.data!.values.isNotEmpty ) {
        var vals = value.data!.values;
        for (var element in vals) {
          var ws = CollaborativeArea().deserialize(element);
          if (ws.id == null) { continue; }
          workspaces[ws.id!] = ws;
        }
      }
    }).catchError((e) => print(e));    
  }

  static CollaborativeArea? getCollaborativeArea(String id) {
    return workspaces[id];
  }

  static Future<void> deleteCollaborativeArea(String id) async {
    if (workspaces.containsKey(id) && workspaces.length == 1) { return; }
    workspaces.remove(id);
    await _service.delete(null, id, {});
  }

  static Future<void> createCollaborativeArea(BuildContext context, String name) async {
    Workspace n = Workspace(name: name);
    await _service.post(context, n.serialize(), {}).then((value) {
      if (value.data != null) {
        workspaces[value.data!.id!] = value.data!;
        endDrawerKey.currentState?.setState(() {});
        CatalogFactory.key.currentState?.setState(() {});
        CatalogItemFactory.key.currentState?.setState(() {});
        WorkflowFactory.key.currentState?.setState(() {});
      }
    });
  }

  static void changeWorkspaceByName(String name) {
    var id = workspaces.entries.firstWhere((element) => element.value.name == "${name}_workspace").key;
    changeWorkspace(id);
  }

  static void changeWorkspace(String id) {
    _service.put(null, id, { "active" : true }, {});
    endDrawerKey.currentState?.setState(() {});
    CatalogFactory.key.currentState?.setState(() {});
    CatalogItemFactory.key.currentState?.setState(() {});
    WorkflowFactory.key.currentState?.setState(() {});
  }

  static List<WorkSpaceItem> getCollaborativeAreasIDS() {
    List<WorkSpaceItem> res = [];
    for (var element in workspaces.entries) {
      res.add(WorkSpaceItem(id: element.key, name: element.value.name));
    }
    return res;
  }
}