import 'dart:math';

import 'package:flutter/material.dart';
import 'package:oc_front/core/sections/end_drawer.dart';
import 'package:oc_front/core/sections/header/header.dart';
import 'package:oc_front/models/resources/compute.dart';
import 'package:oc_front/models/resources/data.dart';
import 'package:oc_front/models/resources/processing.dart';
import 'package:oc_front/models/resources/resources.dart';
import 'package:oc_front/models/resources/storage.dart';
import 'package:oc_front/models/resources/workflow.dart';
import 'package:oc_front/models/response.dart';
import 'package:oc_front/models/workspace.dart';
import 'package:oc_front/core/services/specialized_services/workspace_service.dart';
import 'package:oc_front/pages/catalog.dart';
import 'package:oc_front/pages/catalog_item.dart';
import 'package:oc_front/pages/workflow.dart';

class WorkSpaceItem {
  String? id;
  String? name;
  WorkSpaceItem({this.id, this.name});
}

class WorkspaceLocal {
  static String? current;
  static Map<String, Workspace> workspaces = {};
  static final WorkspaceService _service = WorkspaceService();
  static List<AbstractItem> items = [];

  static Future<void> init(BuildContext? context, bool changeCurrent) async {
    WorkspaceLocal.createWorkspace("default workspace", null);
    var value = await _service.all(context);
    if (value.data != null && value.data!.values.isNotEmpty ) {
      var vals = value.data!.values;
      for (var element in vals) {
        var ws = Workspace().deserialize(element);
        if (ws.id == null) { continue; }
        workspaces[ws.id!] = ws;
        if (current == null) {
          current ??= ws.id;
          workspaces[current!] = ws; 
          _service.put(context, ws.id!, { "active" : true }, {});
        }
        if (ws.active == true && changeCurrent) {
          current = ws.id;
        }
        fill();      
        HeaderConstants.headerKey.currentState?.setState(() {});
      }
    }
  }

  static void fill() {
    items = [];
    if (workspaces[current] != null) {
              for (var element in workspaces[current]!.datas) { 
                if (WorkspaceLocal.hasItemByID(element.getID())) { continue; }
                items.add(element);
              }
              for (var element in workspaces[current]!.processings) { 
                if (WorkspaceLocal.hasItemByID(element.getID())) { continue; }
                items.add(element);  } 
              for (var element in workspaces[current]!.computes) { 
                if (WorkspaceLocal.hasItemByID(element.getID())) { continue; }
                items.add(element);
              }
              for (var element in workspaces[current]!.storages) { 
                if (WorkspaceLocal.hasItemByID(element.getID())) { continue; }
                items.add(element); 
              } 
              for (var element in workspaces[current]!.workflows) { 
                if (WorkspaceLocal.hasItemByID(element.getID())) { continue; }
                items.add(element);
              } 
    }
  }

  static Workspace? getCurrentWorkspace() {
    return workspaces[current];
  }

  static bool hasWorkspace(String workspaceName) {
    for (var element in workspaces.values) {
      if (element.name == workspaceName) { return true; }
    }
    return false;
  }

  static Future<void> deleteWorkspace(String id) async {
    if (workspaces.containsKey(id) && workspaces.length == 1) { return; }
    workspaces.remove(id);
    await _service.delete(null, id, {});
    current = workspaces.keys.first;
  }

  static Future<void> createWorkspace(String name, BuildContext? context) async {
    Workspace n = Workspace(name: name);
    await _service.post(null, n.serialize(), {}).then((value) {
      if (value.data != null) {
        workspaces[value.data!.id!] = value.data!;
        current = value.data!.id;
        fill();
        endDrawerKey.currentState?.setState(() {});
        CatalogFactory.key.currentState?.setState(() {});
        CatalogItemFactory.key.currentState?.setState(() {});
        WorkflowFactory.key.currentState?.setState(() {});
        HeaderConstants.headerKey.currentState?.setState(() {});
      }
    }).catchError( (e) {});
  }

  static void changeWorkspaceByName(String name) {
    try {
      var id = workspaces.entries.firstWhere((element) => element.value.name == "${name}_workspace").key;  
      changeWorkspace(id);
    } catch (e) {} 
  }

  static void changeWorkspace(String id) {
    _service.put(null, id, { "active" : true }, {});
    current = id;
    fill();
    HeaderConstants.headerKey.currentState?.setState(() {});
    endDrawerKey.currentState?.setState(() {});
    CatalogFactory.key.currentState?.setState(() {});
    CatalogItemFactory.key.currentState?.setState(() {});
    WorkflowFactory.key.currentState?.setState(() {});
  }

  static String getWorkspaceName(String id) {
    return workspaces[id]?.name ?? "";
  }

  static List<WorkSpaceItem> getWorkspacesIDS() {
    List<WorkSpaceItem> res = [];
    for (var element in workspaces.entries) {
      res.add(WorkSpaceItem(id: element.key, name: element.value.name));
    }
    return res;
  }

  static List<Shallow> getWorkspacesShallow() {
    List<Shallow> res = [];
    for (var element in workspaces.values) {
      res.add(Shallow(id: element.id ?? "", name: element.name ?? ""));
    }
    return res;
  }

  static List<AbstractItem> byWorkspace(String id) {
    List<AbstractItem> d = [];
    var w = workspaces[id]!;
    d = [ ...d, ...w.datas.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
    d = [ ...d, ...w.computes.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
    d = [ ...d, ...w.processings.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
    d = [ ...d, ...w.storages.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
    d = [ ...d, ...w.workflows.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
    return d;
  }
  
  static List<AbstractItem> byTopic(String topic, bool all) {
    if (all) {
      List<AbstractItem> d = [];
      for (var w in workspaces.values) {
        d = [ ...d, ...w.datas.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.computes.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.processings.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.storages.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.workflows.where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
      }
      return d.where((element) => element.topic.toString() == topic).toList();
    }
    return WorkspaceLocal.items.where((element) => element.topic.toString() == topic).toList();
  }

  static AbstractItem? getItem(String id, bool all) {
    if (all) {
      List<AbstractItem> d = [];
      for (var w in workspaces.values) {
        d = [ ...d, ...w.datas.where((element) => element.id.toString() == id).where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.computes.where((element) => element.id.toString() == id).where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.processings.where((element) => element.id.toString() == id).where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.storages.where((element) => element.id.toString() == id).where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
        d = [ ...d, ...w.workflows.where((element) => element.id.toString() == id).where((element) => !d.map( (d2) => d2.id).contains(element.id)) ];
      }
      return d.isEmpty ? null : d.first;
    }
    var found = WorkspaceLocal.items.where((element) => element.id.toString() == id);
    return found.isEmpty ? null : found.first;
  }

  static void addItem(AbstractItem item) {
    if (!WorkspaceLocal.hasItem(item) && workspaces[current] != null && workspaces[current]!.id != null) { 
      items.add(item); 
      if (item.topic == "data") { workspaces[current]!.datas.add(item as DataItem); }
      if (item.topic == "processing") {  workspaces[current]!.processings.add(item as ProcessingItem); }
      if (item.topic == "compute") { workspaces[current]!.computes.add(item as ComputeItem); }
      if (item.topic == "storage") { workspaces[current]!.storages.add(item as StorageItem); }
      if (item.topic == "workflow") { workspaces[current]!.workflows.add(item as WorkflowItem); }
      try { 
        _service.put(null, workspaces[current]!.id!, workspaces[current]!.serialize(), {});
      } catch (e) { 
        _service.put(null, item.id ?? "", workspaces[current]!.serialize(), {});
      }
    }
  }
  static void removeItem(AbstractItem item) {
    items = items.where((element) => element.name != item.name).toList();
    if (item.topic == "data") { workspaces[current]!.datas.removeWhere( (e) => e.id == item.id); }
    if (item.topic == "processing") { workspaces[current]!.processings.removeWhere( (e) => e.id == item.id); }
    if (item.topic == "compute") { workspaces[current]!.computes.removeWhere( (e) => e.id == item.id); }
    if (item.topic == "storage") { workspaces[current]!.storages.removeWhere( (e) => e.id == item.id); }
    if (item.topic == "workflow") { workspaces[current]!.workflows.removeWhere( (e) => e.id == item.id); }
    try { 
      _service.put(null, workspaces[current]!.id!, workspaces[current]!.serialize(), {});
    } catch (e) { 
      _service.put(null, item.id ?? "", workspaces[current]!.serialize(), {});
    }
  }
  static bool hasItemByID(String id) { 
    return items.where((element) => element.id == id).isNotEmpty;
  }
  static bool hasItem(AbstractItem item) { 
    return items.where((element) => element.id == item.id).isNotEmpty;
  }
  static void clear() => items.clear();
}