import 'package:flutter/material.dart';
import 'package:oc_front/core/conf/conf_reader.dart';
import 'package:oc_front/models/abstract.dart';
import 'package:oc_front/models/response.dart';
import 'package:oc_front/core/services/api_service.dart';

abstract class AbstractService<T extends SerializerDeserializer> {
  abstract APIService<T> service;
  abstract String subPath;
  var conf = AppConfig();

  Future<APIResponse<T>> search(
      BuildContext? context, List<String> words, Map<String, dynamic> params) {
    throw UnimplementedError();
  }

  Future<APIResponse<RawData>> all(BuildContext? context) {
    return service.raw(subPath, null, "get");
  }

  Future<APIResponse<T>> get(BuildContext? context, String id) {
    return service.get("$subPath$id", true, context);
  }

  Future<APIResponse<T>> post(BuildContext? context, Map<String, dynamic> body,
      Map<String, String> params) {
    return service.post(subPath, body, context);
  }

  Future<APIResponse<T>> put(BuildContext? context, String id,
      Map<String, dynamic> body, Map<String, String> params) {
    return service.put("$subPath$id", body, context);
  }

  Future<APIResponse<T>> delete(
      BuildContext? context, String id, Map<String, String> params) {
    return service.delete("$subPath$id", context);
  }
}
