import 'package:oc_front/models/abstract.dart';
import 'package:oc_front/models/resources/compute.dart';
import 'package:oc_front/models/resources/data.dart';
import 'package:oc_front/models/resources/processing.dart';
import 'package:oc_front/models/resources/storage.dart';
import 'package:oc_front/models/resources/workflow.dart';
import 'package:oc_front/models/response.dart';

class Workspace extends SerializerDeserializer<Workspace> implements ShallowData {
  String? id;
  String? name;
  bool? active;
  List<DataItem> datas;
  List<ComputeItem> computes;
  List<StorageItem> storages;
  List<ProcessingItem> processings;
  List<WorkflowItem> workflows;
  String? shared;

  Workspace({
    this.id,
    this.name,
    this.active = false,
    this.workflows = const [],
    this.datas = const [],
    this.computes = const [],
    this.storages = const [],
    this.processings = const [],
    this.shared,
  });

  @override String getID() => id ?? "";
  @override String getName() => name ?? "";

  @override deserialize(dynamic json) {
    try { json = json as Map<String, dynamic>;
    } catch (e) { return Workspace(); }   
    return Workspace(
      id: json.containsKey("id") ? json["id"] : null,
      shared: json["shared"],
      name: json.containsKey("name") ? json["name"] : null,
      active: json.containsKey("active") ? json["active"] : false,
      processings: json.containsKey("processing_resources") ? fromListJson(json["processing_resources"], ProcessingItem()) : [],
      storages: json.containsKey("storage_resources") ? fromListJson(json["storage_resources"], StorageItem()) : [],
      computes: json.containsKey("compute_resources") ? fromListJson(json["compute_resources"], ComputeItem()) : [],
      datas: json.containsKey("data_resources") ? fromListJson(json["data_resources"], DataItem()) : [],
      workflows: json.containsKey("workflow_resources") ? fromListJson(json["workflow_resources"], WorkflowItem()) : []
    );
  }
  @override Map<String, dynamic> serialize() { 
    return {
      "id": id,
      "name": name,
      "processings": processings.map((e) => e.id).toList(),
      "storages": storages.map((e) => e.id).toList(),
      "computes": computes.map((e) => e.id).toList(),
      "datas": datas.map((e) => e.id).toList(),
      "workflows": workflows.map((e) => e.id).toList(),
    };
  }
}