import 'package:oc_front/main.dart';
import 'package:flutter/material.dart';
import 'package:oc_front/models/resources/resources.dart';
import 'package:oc_front/core/services/perms_service.dart';
import 'package:oc_front/widgets/forms/credentials_forms.dart';
import 'package:oc_front/widgets/forms/sub_keys_forms.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:oc_front/widgets/inputs/sub_text_input.dart';
import 'package:oc_front/widgets/forms/container_forms.dart';
import 'package:oc_front/widgets/inputs/sub_dropdown_input%20.dart';

// ignore: must_be_immutable
class ResourceFormsWidget extends StatefulWidget {
  AbstractItem item;
  Dashboard dash;
  String elementID;
  ResourceFormsWidget ({ super.key, required this.item, required this.dash, required this.elementID });
  @override ResourceFormsWidgetState createState() => ResourceFormsWidgetState();
}

class ResourceFormsWidgetState extends State<ResourceFormsWidget> {
  List<Widget> getWidgets(Map<String, dynamic> infos) {
    List<Widget> widgets = [];
    for (var key in infos.keys) {
      if (infos[key] == null && infos[key] != 0) { continue; }
      if (infos[key] is Map) { widgets.addAll(getWidgets(infos[key]));
      } if (infos[key] is List) {
        if (infos[key].isEmpty) { continue; }
        widgets.add(SubTextInputWidget(subkey: key.replaceAll("_", " "), width: 180, empty: false, readOnly: true,
              change: (value) {}, initialValue: (infos[key] as List<dynamic>).join(",") ));
      } else if (infos[key] is Map) {
          widgets.addAll(getWidgets(infos[key] as Map<String, dynamic>));
      } else {
        widgets.add(SubTextInputWidget(subkey: key.replaceAll("_", " "), width: 180, empty: false, readOnly: true,
          change: (value) {}, initialValue: infos[key] is List ? infos[key].join(",") : "${infos[key]}" ));
      }
    }
    if (widgets.isNotEmpty) {
      widgets.add(const SizedBox( width: 200, height: 15) );
    }
    return widgets;
  }

  Widget getInputAndOutputVariableForms(bool readOnly) {
    List<Widget> res = [];
    res.add(SubKeysMapFormsWidget(dash: widget.dash, type: SubMapFormsType.INPUT, item: widget.item, elementID: widget.elementID, 
      readOnly: readOnly));
    res.add(SubKeysMapFormsWidget(dash: widget.dash, type: SubMapFormsType.OUTPUT, item: widget.item, elementID: widget.elementID, 
      readOnly: readOnly));
    res.add(SubKeysMapFormsWidget(dash: widget.dash, type: SubMapFormsType.ENV, item: widget.item, elementID: widget.elementID, 
        readOnly: readOnly));
    return Column( children: res );
  }


  @override Widget build(BuildContext context) {
    print("STATE");
    List<Widget> instancesCat = [];
    List<Widget> childrenReadOnly = getWidgets(widget.item.infos());
    List<DropdownMenuItem<String>> dpItems = [];
    for (var (i, instance) in widget.item.instances.indexed) {
      dpItems.add(DropdownMenuItem(value: '$i', child: Text('${instance.name}', overflow: TextOverflow.ellipsis,)));
    }
    if (dpItems.isNotEmpty) {
      childrenReadOnly.add(Padding( padding: const EdgeInsets.only(top: 20),
        child : Container(
          width: 200, decoration: const BoxDecoration( border: Border(bottom: BorderSide(color: Colors.grey))),
        )));
      childrenReadOnly.add(Padding(padding: const EdgeInsets.only(bottom: 15), child: 
        SubDropdownInputWidget( dropdownMenuEntries: dpItems, subkey: "", width: 180, empty: false, 
          initialValue: "${widget.item.selectedInstance}", change: (value) {
            if (value != null) { setState(() { widget.item.selectedInstance = int.parse(value); }); }
          },
        ))
      );
      if (widget.item.instances.length > (widget.item.selectedInstance)) {
        childrenReadOnly.addAll(getWidgets(widget.item.instances[(widget.item.selectedInstance)].infos()));
      }
    }
    instancesCat.add(ContainerFormsWidget(dash: widget.dash, item: widget.item, elementID: widget.elementID));
    instancesCat.add(CredentialsFormsWidget(dash: widget.dash, item: widget.item, elementID: widget.elementID));
    if (instancesCat.isNotEmpty) {
      instancesCat.add(Container(
          width: 200, decoration: const BoxDecoration( border: Border(bottom: BorderSide(color: Colors.grey))),
        ));
    }
    bool readOnly = !PermsService.getPerm(Perms.WORKFLOW_EDIT);
    // missing env input and output variables
    return SizedBox( height: getHeight(context) - 330, child: SingleChildScrollView( child:  Column( 
      children: [ getInputAndOutputVariableForms(readOnly), ...childrenReadOnly,  ...instancesCat,  ])));
  }
}