import 'package:flutter/material.dart';
import 'package:flutter_advanced_switch/flutter_advanced_switch.dart';
import 'package:oc_front/main.dart';
import 'package:oc_front/models/resources/resources.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:oc_front/models/workflow.dart';
import 'package:oc_front/widgets/inputs/sub_text_input.dart';

// ignore: must_be_immutable
class StorageProcessingLinkFormsWidget extends StatefulWidget {
  Dashboard dash;
  ArrowPainter item;
  bool readOnly = false;
  StorageProcessingLinkFormsWidget ({ super.key, required this.dash, required this.item, this.readOnly = false });
  @override StorageProcessingLinkFormsWidgetState createState() => StorageProcessingLinkFormsWidgetState();
}

class StorageProcessingLinkFormsWidgetState extends State<StorageProcessingLinkFormsWidget> {

  List<Param> getParams(String fromID, String toID) {
    List<Param> arr = [];
    var els = widget.dash.elements.where( (e) => fromID.contains(e.id) || toID.contains(e.id));
    for (var element in els) {
      var g = GraphItem();
      g.fromDashboard(element.serialize());
      var e = g.getElement();
      if (e == null) { continue; }
      if (e.getSelectedInstance() != null) {
        for (var env in e.getSelectedInstance()!.env) {
          if (env.name?.contains("LINK") ?? true) { continue; }
          arr.add(env);
        }
      }
    }
    return arr.where( (e) => e.name != null && e.value != null).toList();
  }


  @override Widget build(BuildContext context) {
    List<Widget> children = [
      const Padding( padding: const EdgeInsets.only(top: 10),
          child: Text("<ENV VARIABLES>", 
            style: const TextStyle(fontSize: 13, fontWeight: FontWeight.bold), textAlign: TextAlign.center)),
    ];
    var params = getParams(widget.item.fromID, widget.item.toID);
    widget.item.env = params.map( (e) => e.serialize()).toList();
    for (var param in params) {
      children.add(SubTextInputWidget(subkey: param.name ?? "", width: 180, empty: false, readOnly: true,
          change: (value) {}, initialValue: param.value ?? "", copyLabel: true));
    }
    children.add(Container(width: 200, padding: const EdgeInsets.only(top: 10, bottom: 10),
          decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1)))));
    List<Widget> inf = [];
    int count = 0;
    for(var info in widget.item.infos) {
      count++;
      inf.add(Padding( padding: const EdgeInsets.only(top: 10, bottom: 5), 
        child : Row( crossAxisAlignment: CrossAxisAlignment.center, children: [
            Padding(padding: const EdgeInsets.only(left: 10, right: 10), child: Text("N°$count")),
            Container(width: 140, decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1))))
          ])));
      for (var key in (info as Map<String, dynamic>).keys) {
          if (info[key] is bool) {
            inf.add(AdvancedSwitch(
              width: 150.0,
              height: 25.0,
              initialValue: info[key] as bool? ?? false,
              enabled: !widget.readOnly,
              activeChild: Text(key, style: const TextStyle(color: Colors.white)),
              inactiveChild: Text(key == "write" ? "read" : "no $key", style: const TextStyle(color: Colors.white)),
              activeColor: Colors.green,
              inactiveColor: redColor,
              onChanged: (v) {
                try { 
                  info[key] = v;
                  Future.delayed(const Duration(seconds: 1), () {
                    for (var a in widget.dash.arrows) {
                      if (a.fromID == widget.item.fromID && a.toID == widget.item.toID) {
                        a.infos = widget.item.infos;
                      }
                    }
                    widget.dash.saveDash(widget.dash.id, context);
                  });
                } catch (e) { /* */ }
                
              },
            ));
          } else if (info[key] is String?) {
            inf.add(SubTextInputWidget(subkey: key, width: 180, empty: false, change: (v) {
              try {
                info[key] = v;
                Future.delayed(const Duration(seconds: 1), () {
                  if (info[key] != v) { return; }
                  for (var a in widget.dash.arrows) {
                    if (a.fromID == widget.item.fromID && a.toID == widget.item.toID) {
                      a.infos = widget.item.infos;
                    }
                  }
                  widget.dash.saveDash(widget.dash.id, context);
                });
              } catch (e) {
                print(e);
              }
                
            }, initialValue: "${info[key] ?? ""}", readOnly: false, noLabel: false));
          }
          
          
      }
    }
    return SizedBox( height: getHeight(context) - 230, child: SingleChildScrollView( child: Column(children: [
      ...children,
      Row( mainAxisAlignment: MainAxisAlignment.center, children: [
              InkWell( onTap: () {
                  widget.item.infos.add(<String, dynamic> {
                            "write": false,
                            "source": null,
                            "destination": null,
                            "filename": null,
                  });
                  setState(() {
                    widget.dash.saveDash(widget.dash.id, context);
                  });
                }, child: Container( margin: const EdgeInsets.only(top: 10),
                    decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), 
                    border: Border.all(color: Colors.grey, width: 1)),
                    width: 125, height: 30,
                    child: const Row( mainAxisAlignment: MainAxisAlignment.center,
                      children: [ Padding( padding: EdgeInsets.only(right: 5), child: Icon(Icons.add, color: Colors.black,)), 
                      Text("link infos",
                        style: TextStyle( color: Colors.black))]),
                  ),  
              ),
              InkWell( onTap: () {
                      if (widget.item.infos.isEmpty) { return; }
                      setState(() { 
                        widget.item.infos.removeLast();
                        widget.dash.saveDash(widget.dash.id, context); 
                      });
                    }, child:
                    Container( margin: const EdgeInsets.only(left: 5, top: 10),
                      decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), 
                        border: Border.all(color: widget.item.infos.isEmpty ? Colors.grey : Colors.black , width: 1)),
                      width: 50, height: 30,
                      child: Row( mainAxisAlignment: MainAxisAlignment.center,
                        children: [ Icon( Icons.delete, size: 18,
                          color: widget.item.infos.isEmpty ? Colors.grey : Colors.black ) ]),
                    )
                  ), 
            ]),
          ...inf
        ] ) ));
  }
}