
import 'package:flutter/material.dart';
import 'package:flutter_advanced_switch/flutter_advanced_switch.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:oc_front/main.dart';
import 'package:oc_front/models/resources/resources.dart';
import 'package:oc_front/models/workflow.dart';
import 'package:oc_front/widgets/forms/sub_add_forms.dart';
import 'package:oc_front/widgets/inputs/sub_text_input.dart';
enum SubMapFormsType { 
  ENV, 
  INPUT,
  OUTPUT 
}
// ignore: must_be_immutable
class SubKeysMapFormsWidget extends StatefulWidget {
  bool readOnly = false;
  AbstractItem item;
  Dashboard dash; 
  bool empty = false;
  SubMapFormsType type;
  String elementID = "";
  SubKeysMapFormsWidget({ super.key, required this.dash, required this.type, this.readOnly = false,
    this.empty = false, required this.item, required this.elementID });
  @override SubKeysMapFormsWidgetState createState() => SubKeysMapFormsWidgetState();
}
class SubKeysMapFormsWidgetState extends State<SubKeysMapFormsWidget> {
  bool _save = false;

  List<Param> getParams(String fromID, String toID) {
    List<Param> arr = [];
    var els = widget.dash.elements.where( (e) => fromID.contains(e.id) || toID.contains(e.id));
    for (var element in els) {
      var g = GraphItem();
      g.fromDashboard(element.serialize());
      var e = g.getElement();
      if (e != null && e.getSelectedInstance() != null) {
        arr = [...arr, ...e.getSelectedInstance()!.env];
      }
    }
    return arr;
  }
  List<Widget> getInstanceInOutput(AbstractItem? item, SubMapFormsType type) {
    List<Widget> children = [];
    if (item == null || item.getSelectedInstance() == null) {
      return [];
    } 
    AbstractInstance instance = item.getSelectedInstance()!;
    List<Param> params = widget.type == SubMapFormsType.INPUT ? instance.inputs : ( widget.type == SubMapFormsType.OUTPUT ? instance.outputs : instance.env);
    for ( var param in params) {
      if (!param.readOnly) { continue; }
      dynamic env;
      if (param.value == null && param.attr != null) { 
        var s = item.serialize();
        s.addAll(item.getSelectedInstance()?.serialize() ?? {});
        env = s[param.attr!];
      } else {  env = param.value; }
      _save = true;
      if (env is bool) {
        var ctrl = ValueNotifier<bool>(env);
        children.add(AdvancedSwitch(
              width: 150.0,
              height: 25.0,
              controller: ctrl,
              enabled: false,
              activeChild: Text(param.name!, style: const TextStyle(color: Colors.white)),
              inactiveChild: Text("no ${param.name!}", style: const TextStyle(color: Colors.white)),
              activeColor: Colors.green,
              inactiveColor: redColor,
              onChanged: (value) {},
        ));
      } else {
        children.add(SubTextInputWidget( subkey: param.name!, width: 180, empty: false, change: (value) { }, 
                     initialValue: env, readOnly: true, noLabel: false, copyLabel: true ));
      }
    }
    return children;
  }

  @override Widget build(BuildContext context) {
    List<Widget> children = [];
    var newwidget = getInstanceInOutput(widget.item, widget.type);
    children.add(Column( children : [
        Padding( padding: const EdgeInsets.only(top: 10),
          child: Text("<${widget.type == SubMapFormsType.INPUT ? "INPUT" : (widget.type == SubMapFormsType.OUTPUT ? "OUTPUT" : "")} ENV VARIABLES>", 
            style: const TextStyle(fontSize: 13, fontWeight: FontWeight.bold), textAlign: TextAlign.center)),
        ...newwidget,
        SubAddFormsWidget(dash: widget.dash, item: widget.item, elementID: widget.elementID, type: widget.type),
        Container(width: 200, padding: const EdgeInsets.only(top: 10, bottom: 10),
          decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1)))),
    ]));
    if (_save) {
      widget.dash.saveDash(widget.dash.id, context);
    }
    return Column( children : children ); 
  }
}