// ignore_for_file: avoid_positional_boolean_parameters

import 'dart:io';
import 'dart:convert';
import 'package:flutter_flow_chart/src/flow_chart_left_menu.dart';
import 'package:uuid/uuid.dart';
import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:flutter_flow_chart/src/flow_chart_menu.dart';
import 'package:flutter_flow_chart/src/flow_chart_selected_menu.dart';
import 'package:flutter_flow_chart/src/ui/segment_handler.dart';

/// Listener definition for a new connection
typedef ConnectionListener = void Function(
  FlowElement srcElement,
  FlowElement destElement,
);

/// Class to store all the scene elements.
/// This also acts as the controller to the flow_chart widget
/// It notifies changes to [FlowChart]
//
class Dashboard extends ChangeNotifier {
  GlobalKey<FlowChartSelectedMenuState> selectedMenuKey = GlobalKey<FlowChartSelectedMenuState>();
  GlobalKey<FlowChartLeftMenuState> selectedLeftMenuKey = GlobalKey<FlowChartLeftMenuState>();
  GlobalKey<FlowChartMenuState> chartMenuKey = GlobalKey<FlowChartMenuState>();
  GlobalKey<ChartWidgetState> chartKey = GlobalKey<ChartWidgetState>();
  GlobalKey<FlowChartState> flutterChartKey = GlobalKey<FlowChartState>();
  bool inDialog = false;
  List<Map<String, dynamic>> tempHistory = [];
  List<Map<String, dynamic>> history = [];

  Map<String, dynamic> info = {};
  String? id;
  String name = "";
  bool isMenu = true;
  bool isInfo = true;
  bool isOpened = false;
  Color defaultColor = Colors.black;
  Map<String, dynamic>? loadedGraph;
  double defaultStroke = 1.7;
  double defaultDashSpace = 0;
  double defaultDashWidth = 0;
  double defaultBackWidth = 10;
  double defaultForwardWidth = 10;
  Future<void> Function(String? id, BuildContext? context)? save;
  List<Widget> Function(FlowData? obj, String id)? infoItemWidget;
  Widget Function(ArrowPainter item)? infoLinkWidget;
  List<Widget> Function()? infoWidget;
  FlowData? Function(Map<String, dynamic> json)? transformToData;
  bool addChange = false;
  bool shouldSave = true;
  Color dashColor = Colors.grey;
  Color midDashColor = Colors.grey.shade300;

  /// The dashboard position
  /// This is used to move the dashboard on the screen
  double widthOffset = 0;
  double heightOffset = 0;
  List<bool Function(Dashboard)> saveRules = [];
  List<List<ArrowPainter> Function(Dashboard)> arrowStyleRules = [];
  
  String? error;
  
  Dashboard({
    this.id,
    this.widthOffset = 0,
    this.heightOffset = 0,
    this.transformToData,
    required this.name,
    this.save,
    this.dashColor = Colors.grey,
    this.midDashColor = Colors.blueGrey,
    Offset? handlerFeedbackOffset,
    this.isMenu = true,
    this.defaultDashSpace = 0,
    this.defaultDashWidth = 0,
    this.blockDefaultZoomGestures = false,
    this.minimumZoomFactor = 0.25,
    this.defaultArrowDirection = ArrowDirection.forward,
    this.defaultArrowStyle = ArrowStyle.curve,
    this.loadedGraph,
    this.infoWidget,
    this.load,
  })  : elements = [],
        _dashboardPosition = Offset.zero,
        dashboardSize = Size.zero,
        gridBackgroundParams = GridBackgroundParams() {
    // This is a workaround to set the handlerFeedbackOffset
    // to improve the user experience on devices with touch screens
    // This will prevent the handler being covered by user's finger
    if (loadedGraph != null) { deserialize(loadedGraph!, false); }
    if (handlerFeedbackOffset != null) {
      this.handlerFeedbackOffset = handlerFeedbackOffset;
    } else {
      if (kIsWeb) {
        this.handlerFeedbackOffset = Offset.zero;
      } else {
        if (Platform.isIOS || Platform.isAndroid) {
          this.handlerFeedbackOffset = const Offset(0, -50);
        } else {
          this.handlerFeedbackOffset = Offset.zero;
        }
      }
    }
    tempHistory = [];
    history = [];
  }
  Future<void> saveDash(String? id, BuildContext? context) async {
    for (var element in saveRules) {
      if (!element(this)) {
        shouldSave = false;
        break;
      }
    }
    if (save != null && shouldSave) {
      save!(id, context);
    }
  }
  Future<void> Function(String cat)? load;

  ///
  factory Dashboard.fromMap(Map<String, dynamic> map) {
    final d = Dashboard(
      name: map['name'] as String,
      isMenu: map['isMenu'] as bool,
      defaultDashSpace: map['defaultDashSpace'] as double? ?? 0,
      defaultDashWidth: map['defaultDashWidth'] as double? ?? 0,
      defaultArrowDirection: ArrowDirection.values[
          map['defaultArrowDirection'] as int? ?? 0],
      defaultArrowStyle: ArrowStyle.values[map['arrowStyle'] as int? ?? 0],
    );
    d..arrows = List<ArrowPainter>.from(
        (map['arrows'] as List<dynamic>).map<ArrowPainter>(
          (x) => ArrowPainter.fromMap(x as Map<String, dynamic>),
        ),
      )
      ..elements = List<FlowElement>.from(
        (map['elements'] as List<dynamic>).map<FlowElement>(
          (x) => FlowElement.fromMap(d, x as Map<String, dynamic>),
        ),
      )
      ..dashboardSize = Size(
        map['dashboardSizeWidth'] as double? ?? 0,
        map['dashboardSizeHeight'] as double? ?? 0,
      );
    for (var f in d.arrowStyleRules) {
      d.arrows = f(d);
    }
    if (map['gridBackgroundParams'] != null) {
      d.gridBackgroundParams = GridBackgroundParams.fromMap(
        map['gridBackgroundParams'] as Map<String, dynamic>,
      );
    }
    d
      ..blockDefaultZoomGestures =
          (map['blockDefaultZoomGestures'] as bool? ?? false)
      ..minimumZoomFactor = map['minimumZoomFactor'] as double? ?? 0.25;

    return d;
  }

  void copyFromMap(Map<String, dynamic> map) {
    defaultArrowStyle = ArrowStyle.values[map['arrowStyle'] as int? ?? 0];
    defaultDashSpace = map['defaultDashSpace'] as double? ?? 0;
    defaultDashWidth = map['defaultDashWidth'] as double? ?? 0;
    defaultArrowDirection = ArrowDirection.values[
          map['defaultArrowDirection'] as int? ?? 0];
    arrows = List<ArrowPainter>.from(
          (map['arrows'] as List<dynamic>).map<ArrowPainter>(
          (x) => ArrowPainter.fromMap(x as Map<String, dynamic>),
        ),
      );
    for (var f in arrowStyleRules) {
      arrows = f(this);
    }
    elements = List<FlowElement>.from(
        (map['elements'] as List<dynamic>).map<FlowElement>(
          (x) => FlowElement.fromMap(this, x as Map<String, dynamic>),
        ),
      );
    dashboardSize = Size(
        map['dashboardSizeWidth'] as double? ?? 0,
        map['dashboardSizeHeight'] as double? ?? 0,
      );

    if (map['gridBackgroundParams'] != null) {
      gridBackgroundParams = GridBackgroundParams.fromMap(
        map['gridBackgroundParams'] as Map<String, dynamic>,
      );
    }
    blockDefaultZoomGestures =
          (map['blockDefaultZoomGestures'] as bool? ?? false);
    minimumZoomFactor = map['minimumZoomFactor'] as double? ?? 0.25;
  }

  ///
  factory Dashboard.fromJson(String source) =>
      Dashboard.fromMap(json.decode(source) as Map<String, dynamic>);
  
  /// The current elements in the dashboard
  List<FlowElement> elements;
  List<FlowElement> get elementSelected => elements.where((element) => element.isSelected).toList();

  Offset _dashboardPosition;

  /// Dashboard size
  Size dashboardSize;

  List<ArrowPainter> arrows = [];
  List<ArrowPainter> get arrowsSelected =>
      arrows.where((element) => element.isSelected).toList();

  /// The default style for the new created arrow
  ArrowStyle defaultArrowStyle;
  ArrowDirection defaultArrowDirection = ArrowDirection.forward;

  /// [handlerFeedbackOffset] sets an offset for the handler when user
  /// is dragging it.
  /// This can be used to prevent the handler being covered by user's
  /// finger on touch screens.
  late Offset handlerFeedbackOffset;

  /// Background parameters.
  GridBackgroundParams gridBackgroundParams;

  ///
  bool blockDefaultZoomGestures;
  double minimumZoomFactor;
  final List<ConnectionListener> _connectionListeners = [];



  Map<String, dynamic> serialize() {
    Map<String, dynamic> d = {};
    Map<String, dynamic> graph = {};
    graph['zoom'] = getZoomFactor();
    graph['elements'] = {};
    for(var el in elements) {
      graph['elements'][el.id] = el.serialize();
    }
    for (var f in arrowStyleRules) {
      arrows = f(this);
    }
    graph['arrows'] = arrows.map((e) => e.serialize()).toList();
    d["id"]=id;
    d["name"]=name;
    d["graph"]=graph;
    return d;
  }

  void deserialize(Map<String, dynamic> graph, bool noHistory) {
    elements = [];
    arrows = [];
    info["shared"] = graph["shared"] ?? [];
    setZoomFactor(graph["graph"]?["zoom"] ?? 1.0);
    for(var el in graph['graph']?['elements'] ?? []) {
      List<ConnectionParams> nexts = [];
      var flow = FlowElement.deserialize(this, el);
      for(var ar in graph['graph']['arrows']) {
        
        if (ar['from']['id'] != flow.id) { continue; }
        nexts.add(ConnectionParams(
          srcElementId: ar['from']['id'],
          destElementId: ar['to']['id'],
          infos: ar['infos'],
          env: ar['env'],
          arrowParams: ArrowParams.fromMap(ar["params"]),
          pivots: [
            Pivot(Offset(ar['from']['x'], (ar['from']['y']))), 
            Pivot(Offset(ar['to']['x'], (ar['to']['y']))) 
          ]
        ));
      }
      flow.next = nexts;
      List<FlowData> arr = [];
      for (var cat in chartKey.currentState?.widget.flowChart.widget.categories ?? []) {
        for (var build in chartKey.currentState!.widget.flowChart.widget.draggableItemBuilder(cat)) {
          arr.add(build);
        }
      }
      if (flow.element != null) {
        flow.kind = ElementKind.widget;
        flow.widget = chartKey.currentState?.widget.flowChart.widget.itemWidget(flow.element!);
      }
      elements.add(flow);
    }
    selectedMenuKey.currentState?.setState(() { });
    chartMenuKey.currentState?.setState(() { });
    if (!noHistory) {
      addToHistory();
    }
  }


  void applyInfos(Map<String, List<dynamic>> elInfos, Map<String, dynamic> graph) {
    for(var id in elInfos.keys) {
      try {
        var element = elements.firstWhere((element) => element.id == id);
        element.element?.addEnv(elInfos[id] ?? []);
      } catch (e) { /* */ }
    }
    for(var ar in graph['graph']['arrows']) {
        var arr = arrows.where((a) => a.fromID == ar['from']['id'] && a.toID == ar['to']['id']).toList();
        for (var a in arr) {
          a.infos = ar['infos'];
          a.env = ar['env'];
        }
      }
    selectedLeftMenuKey.currentState?.setState(() { });
  }

  /// add listener called when a new connection is created
  void addConnectionListener(ConnectionListener listener) {
    _connectionListeners.add(listener);
  }

  /// remove connection listener
  void removeConnectionListener(ConnectionListener listener) {
    _connectionListeners.remove(listener);
  }

  /// set grid background parameters
  void setGridBackgroundParams(GridBackgroundParams params) {
    gridBackgroundParams = params;
    notifyListeners();
  }

  /// set the feedback offset to help on mobile device to see the
  /// end of arrow and not hiding behind the finger when moving it
  void setHandlerFeedbackOffset(Offset offset) {
    handlerFeedbackOffset = offset;
  }

  List<ArrowPainter> getArrowByElementID(String id, bool isinput) {
    return arrows.where((element) => 
      (isinput && element.toID.contains(id)) || (!isinput && element.fromID.contains(id))).toList();
  }

  void addArrows(ArrowPainter f, BuildContext? context) {
    arrows.add(f);
    for (var f in arrowStyleRules) {
      arrows = f(this);
    }
    addChange = true;
    saveDash(id, context);
  }

  void removeArrows(bool Function(ArrowPainter) f, BuildContext? context) {
    arrows.removeWhere((element) => f(element));
    for (var f in arrowStyleRules) {
      arrows = f(this);
    }
    saveDash(id, context);
  }

  void removeElements(bool Function(FlowElement<FlowData>) f, BuildContext? context) {
    elements.removeWhere((element) => f(element));
    saveDash(id, context);
  }

  void clear() {
    elements.clear();
    arrows.clear();
    tempHistory = [];
    history = [];
    info = {};
    notifyListeners();
  }
  bool noHistory = false;
  void addToHistory() {
    if (noHistory) {
      Future.delayed(Duration(seconds: 2), () {
        noHistory = false;
      });
      return;
    }
    if (tempHistory.length >= 50) { tempHistory.removeAt(0); }
    tempHistory.add(toMap());
    history = tempHistory.map((e) => e).toList();
    Future.delayed(Duration(seconds: 1), () {
      chartMenuKey.currentState?.setState(() { });
    });
  }
  bool isBack = false;
  void back() { 
    if (canBack()) {
      noHistory = true;
      tempHistory.removeLast(); 
      if (tempHistory.isNotEmpty) {
        copyFromMap(tempHistory.last);
      }
      chartKey.currentState?.setState(() { });
      chartMenuKey.currentState?.setState(() { });
    }
  }

  bool canBack() {
    return tempHistory.length > 1;
  }

  bool canForward() {
    return tempHistory.length < history.length;
  }

  void forward() {
    if (canForward()) {
      noHistory = true;
      tempHistory.add(history[tempHistory.length]); 
      copyFromMap(tempHistory.last);
      chartKey.currentState?.setState(() { });
      chartMenuKey.currentState?.setState(() { });
    }
  }

  /// set [resizable] element property
  void setElementResizable(
    FlowElement element,
    bool resizable, {
    bool notify = true,
    BuildContext? context
  }) {
    element.isResizing = resizable;
    saveDash(id, context);
    if (notify) notifyListeners();
  }

  FlowElement? getElement(String id, {bool notify = true}) {
    try { return elements.firstWhere((element) {
      return id.contains(element.id);
    }); }
    catch (e) { return null; }
  }

  /// add a [FlowElement] to the dashboard
  void addElement(FlowElement element, BuildContext? context, {bool notify = true}) {
    if (element.id.isEmpty) {
      element.id = const Uuid().v4();
    }
    element.setScale(1, gridBackgroundParams.scale);
    elements.add(element);
    addChange = true;
    saveDash(id, context);
    if (notify) {
      notifyListeners();
    }
  }

  void setElement(FlowElement element, FlowElement set, {bool notify = true}) {
    element = set;
    if (notify) {
      notifyListeners();
    }
  }

  @override
  void notifyListeners() {
    super.notifyListeners();
  }

  /// Set a new [style] to the arrow staring from [src] pointing to [dest].
  /// If [notify] is true the dasboard is refreshed.
  /// The [tension] parameter is used when [style] is [ArrowStyle.segmented] to
  /// set the curve strength on pivot points. 0 means no curve.
  void setArrowStyle(
    FlowElement src,
    FlowElement dest,
    ArrowStyle style, {
    bool notify = true,
    double tension = 1.0,
  }) {
    for (final conn in src.next) {
      if (conn.destElementId == dest.id) {
        conn.arrowParams.style = style;
        conn.arrowParams.tension = tension;
        break;
      }
    }
    if (notify) {
      notifyListeners();
    }
  }

  /// Set a new [style] to the arrow staring from the [handler] of [src]
  /// element.
  /// If [notify] is true the dasboard is refreshed.
  /// The [tension] parameter is used when [style] is [ArrowStyle.segmented] to
  /// set the curve strength on pivot points. 0 means no curve.
  void setArrowStyleByHandler(
    FlowElement src,
    Handler handler,
    ArrowStyle style, {
    bool notify = true,
    double tension = 1.0,
  }) {
    // find arrows that start from [src] inside [handler]
    for (final conn in src.next) {
      if (conn.arrowParams.startArrowPosition == handler.toAlignment()) {
        conn.arrowParams.tension = tension;
        conn.arrowParams.style = style;
      }
    }
    // find arrow that ends to this [src] inside [handler]
    for (final element in elements) {
      for (final conn in element.next) {
        if (conn.arrowParams.endArrowPosition == handler.toAlignment() &&
            conn.destElementId == src.id) {
          conn.arrowParams.tension = tension;
          conn.arrowParams.style = style;
        }
      }
    }

    if (notify) {
      notifyListeners();
    }
  }

  /// find the element by its [id]
  int findElementIndexById(String id) {
    return elements.indexWhere((element) => element.id == id);
  }

  /// find the element by its [id] for convenience
  /// return null if not found
  FlowElement? findElementById(String id) {
    try {
      return elements.firstWhere((element) => element.id == id);
    } catch (e) {
      return null;
    }
  }

  /// find the connection from [srcElement] to [destElement]
  /// return null if not found.
  /// In case of multiple connections, first connection is returned.
  ConnectionParams? findConnectionByElements(
    FlowElement srcElement,
    FlowElement destElement,
  ) {
    try {
      return srcElement.next
          .firstWhere((element) => element.destElementId == destElement.id);
    } catch (e) {
      return null;
    }
  }

  /// find the source element of the [dest] element.
  FlowElement? findSrcElementByDestElement(FlowElement dest) {
    for (final element in elements) {
      for (final connection in element.next) {
        if (connection.destElementId == dest.id) {
          return element;
        }
      }
    }

    return null;
  }

  /// remove all elements
  void removeAllElements(BuildContext? context, {bool notify = true}) {
    elements.clear();
    if (notify) notifyListeners();
    saveDash(id, context);
  }

  /// remove the [handler] connection of [element]
  void removeElementConnection(
    FlowElement element,
    Handler handler, BuildContext? context, {
    bool notify = true,
  }) {
    Alignment alignment;
    switch (handler) {
      case Handler.topCenter:
        alignment = Alignment.topCenter;
      case Handler.bottomCenter:
        alignment = Alignment.bottomCenter;
      case Handler.leftCenter:
        alignment = Alignment.centerLeft;
      case Handler.rightCenter:
        alignment = Alignment.centerRight;
    }

    var isSrc = false;
    for (final connection in element.next) {
      if (connection.arrowParams.startArrowPosition == alignment) {
        isSrc = true;
        break;
      }
    }

    if (isSrc) {
      element.next.removeWhere(
        (handlerParam) =>
            handlerParam.arrowParams.startArrowPosition == alignment,
      );
    } else {
      final src = findSrcElementByDestElement(element);
      if (src != null) {
        src.next.removeWhere(
          (handlerParam) => handlerParam.destElementId == element.id,
        );
      }
    }
    if (notify) notifyListeners();
    saveDash(id, context);
  }

  /// dissect an element connection
  /// [handler] is the handler that is in connection
  /// [point] is the point where the connection is dissected
  /// if [point] is null, point is automatically calculated
  void dissectElementConnection(
    FlowElement element,
    Handler handler, {
    Offset? point,
    bool notify = true,
  }) {
    Alignment alignment;
    switch (handler) {
      case Handler.topCenter:
        alignment = Alignment.topCenter;
      case Handler.bottomCenter:
        alignment = Alignment.bottomCenter;
      case Handler.leftCenter:
        alignment = Alignment.centerLeft;
      case Handler.rightCenter:
        alignment = Alignment.centerRight;
    }

    ConnectionParams? conn;

    var newPoint = Offset.zero;
    if (point == null) {
      try {
        // assuming element is the src
        conn = element.next.firstWhere(
          (handlerParam) =>
              handlerParam.arrowParams.startArrowPosition == alignment,
        );
        if (conn.arrowParams.style != ArrowStyle.segmented) return;

        final dest = findElementById(conn.destElementId);
        newPoint = (dest!
                    .getHandlerPosition(conn.arrowParams.endArrowPosition) +
                element
                    .getHandlerPosition(conn.arrowParams.startArrowPosition)) /
            2;
      } catch (e) {
        // apparently is not
        final src = findSrcElementByDestElement(element)!;
        conn = src.next.firstWhere(
          (handlerParam) => handlerParam.destElementId == element.id,
        );
        if (conn.arrowParams.style != ArrowStyle.segmented) return;

        newPoint = (element
                    .getHandlerPosition(conn.arrowParams.endArrowPosition) +
                src.getHandlerPosition(conn.arrowParams.startArrowPosition)) /
            2;
      }
    } else {
      newPoint = point;
    }

    conn?.dissect(newPoint);

    if (notify && conn != null) {
      notifyListeners();
    }
  }

  /// remove the dissection of the connection
  void removeDissection(Pivot pivot, {bool notify = true}) {
    for (final element in elements) {
      for (final connection in element.next) {
        connection.pivots.removeWhere((item) => item == pivot);
      }
    }
    if (notify) notifyListeners();
  }

  /// remove the connection from [srcElement] to [destElement]
  void removeConnectionByElements(
    FlowElement srcElement,
    FlowElement destElement, {
    bool notify = true,
  }) {
    srcElement.next.removeWhere(
      (handlerParam) => handlerParam.destElementId == destElement.id,
    );
    if (notify) notifyListeners();
  }

  /// remove all the connection from the [element]
  void removeElementConnections(FlowElement element, BuildContext? context, {bool notify = true}) {
    element.next.clear();
    if (notify) notifyListeners();
    saveDash(id, context);
  }

  /// remove all the elements with [id] from the dashboard
  void removeElementById(String id, BuildContext? context, {bool notify = true}) {
    // remove the element
    var elementId = '';
    elements.removeWhere((element) {
      if (element.id == id) {
        elementId = element.id;
        arrows.removeWhere((a) => a.fromID.contains(elementId) || a.toID.contains(elementId));
      }
      return element.id == id;
    });

    // remove all connections to the elements found
    for (final e in elements) {
      e.next.removeWhere((handlerParams) {
        return elementId.contains(handlerParams.destElementId);
      });
    }
    if (notify) notifyListeners();
    saveDash(id, context);
  }

  /// remove element
  /// return true if it has been removed
  bool removeElement(FlowElement element, BuildContext? context, {bool notify = true}) {
    // remove the element
    var found = false;
    final elementId = element.id;
    elements.removeWhere((e) {
      if (e.id == element.id) {
        found = true;

      } 
      return e.id == element.id;
    });
    arrows.removeWhere( (e) => e.fromID.contains(elementId) || e.toID.contains(elementId));
    // remove all connections to the element
    for (final e in elements) {
      e.next.removeWhere(
        (handlerParams) => handlerParams.destElementId == elementId,
      );
    }
    if (notify) notifyListeners();
    saveDash(id, context);
    return found;
  }

  double currentZoom = 1.0;

  /// [factor] needs to be a non negative value.
  /// 1 is the default value.
  /// Giving a value above 1 will zoom the dashboard by the given factor
  /// and vice versa. Negative values will be ignored.
  /// [zoomFactor] will not go below [minimumZoomFactor]
  /// [focalPoint] is the point where the zoom is centered
  /// default is the center of the dashboard
  void setZoomFactor(double factor, {Offset? focalPoint}) {
    if (factor < minimumZoomFactor || gridBackgroundParams.scale == factor) {
      return;
    }

    focalPoint ??= Offset(dashboardSize.width / 2, dashboardSize.height / 2);

    for (final element in elements) {
      // applying new zoom
      element
        ..position = (element.position - focalPoint) /
                gridBackgroundParams.scale *
                factor +
            focalPoint
        ..setScale(gridBackgroundParams.scale, factor);
      for (final conn in element.next) {
        for (final pivot in conn.pivots) {
          pivot.setScale(gridBackgroundParams.scale, focalPoint, factor);
        }
      }
    }

    gridBackgroundParams.setScale(factor, focalPoint);
    currentZoom = factor;
    notifyListeners();
  }

  double getZoomFactor() {
    return currentZoom;
  }

  /// shorthand to get the current zoom factor
  double get zoomFactor {
    return gridBackgroundParams.scale;
  }

  /// needed to know the diagram widget position to compute
  /// offsets for drag and drop elements
  void setDashboardPosition(Offset position) {
    _dashboardPosition = position;
  }

  /// Get the position.
  Offset get position => _dashboardPosition;

  /// needed to know the diagram widget size
  void setDashboardSize(Size size) {
    dashboardSize = size;
  }

  /// make an arrow connection from [sourceElement] to
  /// the elements with id [destId]
  /// [arrowParams] definition of arrow parameters
  void addNextById(
    FlowElement sourceElement,
    String destId,
    ArrowParams arrowParams, {
    bool notify = true,
  }) {
    var found = 0;
    arrowParams.setScale(1, gridBackgroundParams.scale);
    for (var i = 0; i < elements.length; i++) {
      if (elements[i].id == destId) {
        // if the [id] already exist, remove it and add this new connection
        /* sourceElement.next
            .removeWhere((element) => element.destElementId == destId); */
        final conn = ConnectionParams(
          srcElementId: sourceElement.id,
          destElementId: elements[i].id,
          arrowParams: arrowParams,
          pivots: [],
        );
        sourceElement.next.add(conn);
        for (final listener in _connectionListeners) {
          listener(sourceElement, elements[i]);
        }

        found++;
      }
    }

    if (found == 0) {
      return;
    }
    if (notify) {
      notifyListeners();
    }
  }

  //******************************* */
  /// manage load/save using json
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'name': name,
      'isMenu': isMenu,
      'arrows': arrows.map((x) => x.toMap()).toList(),
      'elements': elements.map((x) => x.toMap()).toList(),
      'dashboardSizeWidth': dashboardSize.width,
      'dashboardSizeHeight': dashboardSize.height,
      'gridBackgroundParams': gridBackgroundParams.toMap(),
      'blockDefaultZoomGestures': blockDefaultZoomGestures,
      'minimumZoomFactor': minimumZoomFactor,
      'defaultArrowDirection': defaultArrowDirection.index,
      'defaultDashSpace': defaultDashSpace,
      'defaultDashWidth': defaultDashWidth,
      'arrowStyle': defaultArrowStyle.index,
    };
  }

  ///
  String toJson() => json.encode(toMap());

  ///
  String prettyJson() {
    final spaces = ' ' * 2;
    final encoder = JsonEncoder.withIndent(spaces);
    return encoder.convert(toMap());
  }

  /// recenter the dashboard
  void recenter() {
    final center = Offset(dashboardSize.width / 2, dashboardSize.height / 2);
    gridBackgroundParams.offset = center;
    if (elements.isNotEmpty) {
      final currentDeviation = elements.first.position - center;
      for (final element in elements) {
        element.position -= currentDeviation;
        for (final next in element.next) {
          for (final pivot in next.pivots) {
            pivot.pivot -= currentDeviation;
          }
        }
      }
    }
    notifyListeners();
  }

  /// save the dashboard into [completeFilePath]
  void saveDashboard(String completeFilePath) {
    File(completeFilePath).writeAsStringSync(prettyJson(), flush: true);
  }

  /// clear the dashboard and load the new one
  void loadDashboard(String completeFilePath) {
    final f = File(completeFilePath);
    if (f.existsSync()) {
      elements.clear();
      final source = json.decode(f.readAsStringSync()) as Map<String, dynamic>;

      gridBackgroundParams = GridBackgroundParams.fromMap(
        source['gridBackgroundParams'] as Map<String, dynamic>,
      );
      blockDefaultZoomGestures = source['blockDefaultZoomGestures'] as bool;
      minimumZoomFactor = source['minimumZoomFactor'] as double;
      dashboardSize = Size(
        source['dashboardSizeWidth'] as double,
        source['dashboardSizeHeight'] as double,
      );

      final loadedElements = List<FlowElement>.from(
        (source['elements'] as List<dynamic>).map<FlowElement>(
          (x) => FlowElement.fromMap(this, x as Map<String, dynamic>),
        ),
      );
      elements
        ..clear()
        ..addAll(loadedElements);

      recenter();
    }
  }
}
