import 'dart:convert';
import 'dart:ui';

import 'package:flutter_flow_chart/src/ui/draw_arrow.dart';
import 'package:flutter_flow_chart/src/ui/segment_handler.dart';

/// Connection parameters.
class ConnectionParams {
  ///
  ConnectionParams({
    required this.srcElementId,
    required this.destElementId,
    required this.arrowParams,
    this.env = const [],
    this.infos = const [],
    List<Pivot>? pivots,
  }) : pivots = pivots ?? [];
  List<dynamic> env = [];
  List<dynamic> infos = [];
  ///
  factory ConnectionParams.fromMap(Map<String, dynamic> map) {
    return ConnectionParams(
      srcElementId: map['srcElementId'] as String,
      destElementId: map['destElementId'] as String,
      infos: map['infos'] as List<dynamic>? ?? [],
      env: map['env'] as List<dynamic>? ?? [],
      arrowParams: ArrowParams.fromMap(map['arrowParams'] as Map<String, dynamic>),
      pivots: (map['pivots'] as List?)
              ?.map<Pivot>(
                (pivot) => Pivot.fromMap(pivot as Map<String, dynamic>),
              )
              .toList() ??
          [],
    );
  }

  ///
  factory ConnectionParams.fromJson(String source) =>
      ConnectionParams.fromMap(json.decode(source) as Map<String, dynamic>);

  /// Unique element ID where this connection points.
  final String destElementId;
  final String srcElementId;

  /// Arrow parameters.
  final ArrowParams arrowParams;

  /// List of pivot points for the segmented arrow style.
  final List<Pivot> pivots;

  ///
  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'srcElementId' : srcElementId,
      'destElementId': destElementId,
      'arrowParams': arrowParams.toMap(),
      'pivots': pivots.map((pivots) => pivots.toMap()).toList(),
    };
  }

  /// Divide the connection into segments
  void dissect(Offset point) {
    pivots.add(Pivot(point));
  }

  ///
  String toJson() => json.encode(toMap());
}
