// ignore: directives_ordering
import 'package:uuid/uuid.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_flow_chart/src/ui/draw_arrow.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:flutter_flow_chart/src/flow_chart_menu.dart';
import 'package:flutter_flow_chart/src/ui/element_widget.dart';
import 'package:flutter_flow_chart/src/ui/grid_background.dart';
import 'package:flutter_flow_chart/src/ui/segment_handler.dart';
import 'package:flutter_flow_chart/src/flow_chart_left_menu.dart';
import 'package:flutter_flow_chart/src/flow_chart_selected_menu.dart';


/// Main flow chart Widget.
/// It displays the background grid, all the elements and connection lines
abstract class FlowData {
  String getID();
  String getName();
  String getType();
  double? getWidth();
  double? getHeight();
  void addEnv(List<dynamic> infos);
  Map<String, dynamic> serialize();
  FlowData deserialize(Map<String, dynamic> data);
}

class FlowChart<T extends FlowData> extends StatefulWidget {
  FlowChart({
    required this.dashboard,
    required this.itemWidget,
    super.key,
    this.itemWidgetTooltip,
    this.onElementPressed,
    this.onElementSecondaryTapped,
    this.onElementLongPressed,
    this.onElementSecondaryLongTapped,
    this.onDashboardTapped,
    this.onDashboardSecondaryTapped,
    this.onDashboardLongTapped,
    this.onDashboardSecondaryLongTapped,
    this.onHandlerPressed,
    this.onHandlerSecondaryTapped,
    this.onHandlerLongPressed,
    this.onHandlerSecondaryLongTapped,
    this.onPivotPressed,
    this.onPivotSecondaryPressed,
    this.onScaleUpdate,
    this.onNewConnection,
    this.width = 1000,
    this.height = 1000,
    this.innerMenuWidth = 200,
    this.itemWidth = 80,
    this.categories = const [],
    required this.draggableItemBuilder,
    this.onDashboardAlertOpened,
    this.menuWidget,
    this.current,
    this.menuExtension,
    this.itemLeftBottomBadges,
    this.itemrightTopBadges,
  }) {}
  final String? current;
  final List<String> categories;
  final double width;
  final double height;
  final double innerMenuWidth;

  Widget Function()? menuExtension;
  Widget Function(Dashboard data)? menuWidget;

  double itemWidth = 80;
  double zoom = 1;

  final Widget Function(T data) itemWidget;
  final Widget? Function(T data)? itemLeftBottomBadges;
  final Widget? Function(T data)? itemrightTopBadges;
  final Widget Function(T data)? itemWidgetTooltip;
  final List<T> Function(String cat) draggableItemBuilder;

  final Widget Function(BuildContext constext, Dashboard dash)? onDashboardAlertOpened;
  /// callback for tap on dashboard
  final void Function(BuildContext context, Offset position)? onDashboardTapped;

  /// callback for long tap on dashboard
  final void Function(BuildContext context, Offset position)?
      onDashboardLongTapped;

  /// callback for mouse right click on dashboard
  final void Function(BuildContext context, Offset postision)?
      onDashboardSecondaryTapped;

  /// callback for mouse right click long press on dashboard
  final void Function(BuildContext context, Offset position)?
      onDashboardSecondaryLongTapped;

  /// callback for element pressed
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementPressed;

  /// callback for mouse right click event on an element
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementSecondaryTapped;

  /// callback for element long pressed
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementLongPressed;

  /// callback for right click long press event on an element
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementSecondaryLongTapped;

  /// callback for onclick event of pivot
  final void Function(BuildContext context, Pivot pivot)? onPivotPressed;

  /// callback for secondary press event of pivot
  final void Function(BuildContext context, Pivot pivot)?
      onPivotSecondaryPressed;

  /// callback for handler pressed
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerPressed;

  /// callback for handler right click event
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerSecondaryTapped;

  /// callback for handler right click long press event
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerSecondaryLongTapped;

  /// callback for handler long pressed
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerLongPressed;

  /// callback when adding a new connection
  final ConnectionListener? onNewConnection;

  /// main dashboard to use
  final Dashboard dashboard;

  /// Trigger for the scale change
  final void Function(double scale)? onScaleUpdate;

  @override
  State<FlowChart> createState() => FlowChartState<T>();
}

class HoverMenuController {
  HoverMenuState? currentState;

  void hideSubMenu() {
    currentState?.hideSubMenu();
  }
}
class HoverMenu extends StatefulWidget {
  final Widget title;
  final double? width;
  final List<Widget> items;
  final HoverMenuController? controller;
  bool isHovered = false;


  HoverMenu({
    Key? key,
    required this.title,
    this.items = const [],
    this.width,
    this.controller,
  }) : super(key: key);

  @override
  HoverMenuState createState() => HoverMenuState();
}

class HoverMenuState extends State<HoverMenu> {
  OverlayEntry? _overlayEntry;
  final _focusNode = FocusNode();

  @override
  void initState() {
    super.initState();
    _focusNode.addListener(_onFocusChanged);

    if (widget.controller != null) {
      widget.controller?.currentState = this;
    }
  }

  @override
  void dispose() {
    _focusNode.dispose();
    super.dispose();
  }

  void _onFocusChanged() {
    if (_focusNode.hasFocus) {
      _overlayEntry = _createOverlayEntry();
      Overlay.of(context).insert(_overlayEntry!);
    } else {
      _overlayEntry?.remove();
      _removeOverlay();
    }
  }

  void _removeOverlay() {
    widget.isHovered = false;
  }

  void hideSubMenu() {
    _focusNode.unfocus();
  }

  @override
  Widget build(BuildContext context) {
    return TextButton(
      style: TextButton.styleFrom(
                  elevation: 0.0,
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0)),
                  splashFactory: NoSplash.splashFactory,
                  padding: EdgeInsets.zero,
                  foregroundColor: Colors.transparent,
                  surfaceTintColor: Colors.transparent,
                  backgroundColor: Colors.transparent,
                  shadowColor: Colors.transparent,
                ),
      focusNode: _focusNode,
      onHover: (isHovered) {
        if (isHovered && !widget.isHovered) {
          _focusNode.requestFocus();
          isHovered = true;
        } else {
          _focusNode.unfocus();
          widget.isHovered = false;
        }
      },
      onPressed: () {},
      child: widget.title,
    );
  }

  OverlayEntry _createOverlayEntry() {
    final renderBox = context.findRenderObject() as RenderBox;
    final size = renderBox.size;
    final offset = renderBox.localToGlobal(Offset.zero);

    return OverlayEntry(
      maintainState: true,
      builder: (context) => Positioned(
        left: offset.dx - 300,
        top: offset.dy + size.height,
        width: 300,
        child: TextButton(
            style: TextButton.styleFrom( 
              padding: EdgeInsets.zero,
              shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(0)),
              splashFactory: NoSplash.splashFactory,
              backgroundColor: Colors.transparent,
            ),
            onPressed: () {},
            child: ListView(
                padding: EdgeInsets.zero,
                shrinkWrap: true,
                children: widget.items)),
      ),
    );
  }
}

bool isCtrl = false;
class FlowChartState<T extends FlowData> extends State<FlowChart> {
  var node = FocusNode();
  @override
  void initState() {
    node.requestFocus();
    super.initState();
  }

  @override
  void dispose() {
    node.requestFocus();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (!widget.dashboard.isOpened && widget.onDashboardAlertOpened != null ) {
      if (widget.dashboard.id != null) {
        widget.dashboard.isOpened  = true;
        Future.delayed(const Duration(milliseconds: 100), () => widget.dashboard.load!(widget.dashboard.id!) );    
      } else {
        try {
          Future.delayed(const Duration(milliseconds: 100), () {
            if (!widget.dashboard.inDialog) {
              widget.dashboard.inDialog = true;
              showDialog(
                barrierDismissible: false,
                context: context, builder: (context) {
                return AlertDialog(
                  titlePadding: EdgeInsets.zero,
                  insetPadding: EdgeInsets.zero,
                  backgroundColor: Colors.white,
                  shape: RoundedRectangleBorder( borderRadius: BorderRadius.circular(0)),
                  title: widget.onDashboardAlertOpened!(context, widget.dashboard));
                
              });
            }
          });
        } catch (e) {
          widget.dashboard.id = null;
          widget.dashboard.name = "";
          widget.dashboard.isOpened  = false;
        }
      }
    } else {
      widget.dashboard.isOpened  = true;
    }
    /// get dashboard positi(event is KeyDownEvent || event is KeyRepeatEvent) &&on after first frame is drawn
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      if (mounted) {
        final object = context.findRenderObject() as RenderBox?;
        if (object != null) {
          final translation = object.getTransformTo(null).getTranslation();
          final size = object.semanticBounds.size;
          final position = Offset(translation.x, translation.y);
          widget.dashboard.setDashboardSize(size);
          widget.dashboard.setDashboardPosition(position);
        }
      }
    });
    // disabling default browser context menu on web
    if (kIsWeb) BrowserContextMenu.disableContextMenu();
    return KeyboardListener(
      focusNode: node,
      autofocus: true,
      onKeyEvent: (event) {
        bool change = false;
        if (event.logicalKey == LogicalKeyboardKey.controlLeft && event is KeyDownEvent) {
          print("CTRL true");
          isCtrl = true;
        }
        if (event.logicalKey == LogicalKeyboardKey.controlLeft && event is KeyUpEvent) {
          print("CTRL false");
          isCtrl = false;
        }
        if ((event is KeyDownEvent || event.logicalKey == LogicalKeyboardKey.keyZ) && isCtrl) {
          widget.dashboard.back();
        }
        if ((event is KeyDownEvent || event.logicalKey == LogicalKeyboardKey.keyY) && isCtrl) {
          widget.dashboard.forward();
        }
        if (event is KeyDownEvent && event.logicalKey == LogicalKeyboardKey.add) {
          change = true;
          for (var el in widget.dashboard.elementSelected) {
              widget.dashboard.addElement(FlowElement<T>(
                element: el.element as T,
                dashboard: widget.dashboard,
                id: const Uuid().v4(),
                position: el.position + const Offset(100, 100),
                size: el.size,
                text: el.text,
                widget: el.widget,
                kind: el.kind,
                handlers: el.handlers,
                handlerSize: el.handlerSize,
              ), context);
            }
        }
        if (event is KeyDownEvent && event.logicalKey == LogicalKeyboardKey.delete) {
          change = true;
          widget.dashboard.removeElements((element) {
              if (element.isSelected) {
                widget.dashboard.arrows.removeWhere( (e) => e.toID.contains(element.id) || e.fromID.contains(element.id));
              }
            return element.isSelected;
          }, context);
          for (var arrow in widget.dashboard.arrowsSelected) {
              for (var el in widget.dashboard.elements.where((element) => element.id == arrow.fromID.split("_")[0])) {
                el.next.removeAt(int.parse(arrow.fromID.split("_")[1]));
              }
          }
          widget.dashboard.removeArrows( (el) => el.isSelected, context );
        }
        if (change) {
          DrawingArrow.instance.notifyListeners();
          widget.dashboard.chartKey.currentState?.setState(() {  });
          node.requestFocus();
        }
      }, 
      child: ClipRect(
      child: Stack(
        clipBehavior: Clip.none,
        children: [
          Stack(children: [
            // Draw the grid
            DragTarget<T>(
              builder: (
                BuildContext context,
                List<dynamic> accepted,
                List<dynamic> rejected,
              ) {
                return SizedBox(
                  width: widget.width,
                  height: widget.height,
                  child: ChartWidget<T>(
                    key: widget.dashboard.chartKey,
                    flowChart: this,
                    menuWidget: widget.menuWidget,
                    dashboard: widget.dashboard,
                    onNewConnection: widget.onNewConnection,
                    onDashboardTapped: widget.onDashboardLongTapped,
                    onScaleUpdate: widget.onScaleUpdate,
                    onDashboardSecondaryTapped: widget.onDashboardSecondaryTapped,
                    onDashboardLongTapped: widget.onDashboardLongTapped,
                    onDashboardSecondaryLongTapped: widget.onDashboardSecondaryLongTapped,
                    onElementLongPressed: widget.onElementLongPressed,
                    onElementSecondaryLongTapped: widget.onElementSecondaryLongTapped,
                    onElementPressed: widget.onElementPressed,
                    onElementSecondaryTapped: widget.onElementSecondaryTapped,
                    onHandlerPressed: widget.onHandlerLongPressed,
                    onHandlerLongPressed: widget.onHandlerLongPressed,
                    onPivotSecondaryPressed: widget.onPivotSecondaryPressed,
                ));
            },
            onAcceptWithDetails: (DragTargetDetails<T> details) {
              var e = details.data;
              String newID = const Uuid().v4();
              double ratio = 1;
              if (e.getWidth() != null && e.getHeight() != null) {
                var h = e.getHeight()! < e.getWidth()! ? e.getHeight()! + 20 : e.getHeight()!;
                var w = e.getWidth()! < e.getHeight()! ? e.getWidth()! + 20 : e.getWidth()!;
                ratio = (h / w);
              }
              FlowElement<T> el = FlowElement<T>(
                    dashboard: widget.dashboard,
                    id: newID,
                    element: e,
                    position: details.offset,
                    size: Size(100, 100 * ratio),
                    text: '${widget.dashboard.elements.length}',
                    handlerSize: 15,
                    widget: widget.itemWidget(e),
                    kind: ElementKind.widget,
                    handlers: [
                      Handler.bottomCenter,
                      Handler.topCenter,
                      Handler.leftCenter,
                      Handler.rightCenter,
                    ],
                  );
              widget.dashboard.addElement(el, context);
            },
                      )]
        ),
        widget.dashboard.isMenu ? Positioned(top: 50, child: FlowChartSelectedMenu(
          key: widget.dashboard.selectedMenuKey,
          dashboard: widget.dashboard, 
          height: widget.height - 50) ) : Container(),
        widget.dashboard.isInfo ? Positioned(top: 50, right: 0, child: 
          FlowChartLeftMenu<T>(
            key: widget.dashboard.selectedLeftMenuKey,
            dashboard: widget.dashboard, 
            menuExtension: widget.menuExtension,
            categories: widget.categories,
            height: widget.height,
            innerMenuWidth: widget.innerMenuWidth,
            itemWidth: widget.itemWidth,
            draggableItemBuilder: widget.draggableItemBuilder as List<T> Function(String cat),
            getDraggable: getDraggable,
          )) : Container()
      ])
    ));
  }

  List<Draggable<T>> getDraggable(List<T> items) {
    List<Draggable<T>> res = [];
    double realSize = widget.itemWidth * widget.zoom;
    for (var e in items) {
      GlobalKey<HoverMenuState> hoverKey = GlobalKey<HoverMenuState>();
      res.add(Draggable<T>(
        // Data is the value this Draggable stores.
        data: e,
        onDragStarted: () => hoverKey.currentState?.hideSubMenu(),
        onDragEnd: (d) => node.requestFocus(),
        childWhenDragging: Opacity(opacity: .5, 
        child: Padding( padding: const EdgeInsets.all(10), 
          child: Container( alignment: Alignment.center,
            constraints: BoxConstraints(maxHeight: realSize - 10, maxWidth: realSize - 10), child: widget.itemWidget(e), ))),
        feedback: Container( alignment: Alignment.center, constraints: BoxConstraints(maxHeight: realSize, maxWidth: realSize),
          child: widget.itemWidget(e) ),
        child: InkWell( mouseCursor: SystemMouseCursors.grab, child: Padding( padding: const EdgeInsets.all(10), 
        child: widget.itemWidgetTooltip != null ? HoverMenu( key: hoverKey, width: 400, title: Container(  alignment: Alignment.center,
          constraints: BoxConstraints( maxHeight: realSize - 10, maxWidth: realSize - 10), child: widget.itemWidget(e) ),
          items: [ Container(child: widget.itemWidgetTooltip!(e))  ]
        ) : Container( alignment: Alignment.center,
          constraints: BoxConstraints(maxHeight: realSize - 10, maxWidth: realSize - 10), child: widget.itemWidget(e) 
        )
      ) )));
    }
    return res;
  }
}

/// Widget to draw interactive connection when the user tap on handlers
class DrawingArrowWidget extends StatefulWidget {
  ///
  const DrawingArrowWidget({required this.style, super.key});

  ///
  final ArrowStyle style;

  @override
  State<DrawingArrowWidget> createState() => _DrawingArrowWidgetState();
}

class _DrawingArrowWidgetState extends State<DrawingArrowWidget> {
  @override
  void initState() {
    super.initState();
    DrawingArrow.instance.addListener(_arrowChanged);
  }

  @override
  void dispose() {
    DrawingArrow.instance.removeListener(_arrowChanged);
    super.dispose();
  }

  void _arrowChanged() {
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    if (DrawingArrow.instance.isZero()) return const SizedBox.shrink();
    return CustomPaint(
      painter: ArrowPainter(
        fromID: DrawingArrow.instance.fromID,
        toID: "",
        env: DrawingArrow.instance.env,
        infos: DrawingArrow.instance.infos,
        params: DrawingArrow.instance.params,
        from: DrawingArrow.instance.from,
        to: DrawingArrow.instance.to,
      ),
    );
  }
}

class ChartWidget<T extends FlowData> extends StatefulWidget {
  ChartWidget ({ Key? key,
        required this.flowChart,
        this.onElementPressed,
        this.onElementSecondaryTapped,
        this.onElementLongPressed,
        this.onElementSecondaryLongTapped,
        this.onDashboardTapped,
        this.onDashboardSecondaryTapped,
        this.onDashboardLongTapped,
        this.onDashboardSecondaryLongTapped,
        this.onHandlerPressed,
        this.onHandlerSecondaryTapped,
        this.onHandlerLongPressed,
        this.onHandlerSecondaryLongTapped,
        this.onPivotPressed,
        this.onPivotSecondaryPressed,
        this.onScaleUpdate,
        required this.dashboard,
        this.onNewConnection,
        this.menuWidget,
  }) : super(key: key);

  FlowChartState flowChart;

  Widget Function(Dashboard dash)? menuWidget;

  final void Function(BuildContext context, Offset position)? onDashboardTapped;

  /// callback for long tap on dashboard
  final void Function(BuildContext context, Offset position)?
      onDashboardLongTapped;

  /// callback for mouse right click on dashboard
  final void Function(BuildContext context, Offset postision)?
      onDashboardSecondaryTapped;

  /// callback for mouse right click long press on dashboard
  final void Function(BuildContext context, Offset position)?
      onDashboardSecondaryLongTapped;

  /// callback for element pressed
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementPressed;

  /// callback for mouse right click event on an element
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementSecondaryTapped;

  /// callback for element long pressed
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementLongPressed;

  /// callback for right click long press event on an element
  final void Function(
    BuildContext context,
    Offset position,
    FlowElement element,
  )? onElementSecondaryLongTapped;

  /// callback for onclick event of pivot
  final void Function(BuildContext context, Pivot pivot)? onPivotPressed;

  /// callback for secondary press event of pivot
  final void Function(BuildContext context, Pivot pivot)?
      onPivotSecondaryPressed;

  /// callback for handler pressed
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerPressed;

  /// callback for handler right click event
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerSecondaryTapped;

  /// callback for handler right click long press event
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerSecondaryLongTapped;

  /// Trigger for the scale change
  final void Function(double scale)? onScaleUpdate;

  /// callback for handler long pressed
  final void Function(
    BuildContext context,
    Offset position,
    Handler handler,
    FlowElement element,
  )? onHandlerLongPressed;

  /// callback when adding a new connection
  final ConnectionListener? onNewConnection;

  final Dashboard dashboard;

  @override ChartWidgetState createState() => ChartWidgetState<T>();
}
class ChartWidgetState<T extends FlowData> extends State<ChartWidget> {
  bool hoverImportant = false;
  final segmentedTension = ValueNotifier<double>(1);
  
    @override
  void initState() {
    super.initState();
    widget.dashboard.addListener(_elementChanged);
    if (widget.onScaleUpdate != null) {
      widget.dashboard.gridBackgroundParams.addOnScaleUpdateListener(
        widget.onScaleUpdate!,
      );
    }
    if (widget.onNewConnection != null) {
      widget.dashboard.addConnectionListener(widget.onNewConnection!);
    }
  }

  @override
  void dispose() {
    widget.dashboard.removeListener(_elementChanged);
    if (widget.onScaleUpdate != null) {
      widget.dashboard.gridBackgroundParams.removeOnScaleUpdateListener(
        widget.onScaleUpdate!,
      );
    }
    super.dispose();
  }

  void _elementChanged() {
    if (mounted) setState(() {});
  }

  double _oldScaleUpdateDelta = 0;

  @override Widget build(BuildContext context) {
    final gridKey = GlobalKey();
    var tapDownPos = Offset.zero;
    var secondaryTapDownPos = Offset.zero;
    for (int i = 0; i < widget.dashboard.elements.length; i++) {
      widget.dashboard.elements[i].next.removeWhere((element) =>
                      widget.dashboard.findElementIndexById(element.destElementId) < 0);
    }
    return Stack( children: [
                  Positioned.fill(
                    child: GestureDetector(
                      onTapDown: (details) {
                        hoverImportant = false;
                        tapDownPos = details.localPosition;

                        for (var arr in widget.dashboard.arrows) {
                          if (arr.isLine(tapDownPos)) {
                            hoverImportant = true;
                            for (var arr in widget.dashboard.arrows) {
                              arr.isSelected = false;
                            }
                            arr.isSelected = !arr.isSelected;
                            for (var sel in widget.dashboard.elements) {
                              sel.isSelected = false;
                            }
                            Future.delayed(const Duration(milliseconds: 100), () {
                              widget.dashboard.selectedMenuKey.currentState?.setState(() {});
                              DrawingArrow.instance.notifyListeners();
                            });
                            break;
                          }
                        }
                        if (!hoverImportant) {
                          for (var sel in widget.dashboard.elements) { sel.isSelected = false; }
                          for (var sel in widget.dashboard.arrows) { sel.isSelected = false; }
                          Future.delayed(Duration(milliseconds: 100), () {
                            widget.dashboard.selectedMenuKey.currentState?.setState(() {});
                            DrawingArrow.instance.notifyListeners();
                          });
                        }
                        setState(() {});
                      },
                      onSecondaryTapDown: (details) {
                        secondaryTapDownPos = details.localPosition;
                      },
                      onTap: widget.onDashboardTapped == null
                          ? null
                          : () => widget.onDashboardTapped!(
                                gridKey.currentContext!,
                                tapDownPos,
                              ),
                      onLongPress: widget.onDashboardLongTapped == null
                          ? null
                          : () => widget.onDashboardLongTapped!(
                                gridKey.currentContext!,
                                tapDownPos,
                              ),
                      onSecondaryTap: () {
                        widget.onDashboardSecondaryTapped?.call(
                          gridKey.currentContext!,
                          secondaryTapDownPos,
                        );
                      },
                      onSecondaryLongPress: () {
                        widget.onDashboardSecondaryLongTapped?.call(
                          gridKey.currentContext!,
                          secondaryTapDownPos,
                        );
                      },
                      onScaleUpdate: (details) {
                        if (details.scale != 1) {
                          widget.dashboard.setZoomFactor(
                            details.scale + _oldScaleUpdateDelta,
                            focalPoint: details.focalPoint,
                          );
                        }
                        widget.dashboard.setDashboardPosition(
                          widget.dashboard.position + details.focalPointDelta,
                        );
                        for (var i = 0; i < widget.dashboard.elements.length; i++) {
                          widget.dashboard.elements[i].position += details.focalPointDelta;
                          for (final conn in widget.dashboard.elements[i].next) {
                            for (final pivot in conn.pivots) {
                              pivot.pivot += details.focalPointDelta;
                            }
                          }
                        }
                        widget.dashboard.gridBackgroundParams.offset = details.focalPointDelta;
                        setState(() {});
                      },
                      onScaleEnd: (details) {
                        _oldScaleUpdateDelta = widget.dashboard.zoomFactor - 1;
                      },
                      child: GridBackground(
                        key: gridKey,
                        params: widget.dashboard.gridBackgroundParams,
                      ),
                    ),
                  ),
                  // Draw elements
                  for (int i = 0; i < widget.dashboard.elements.length; i++)
                    ElementWidget<T>(
                      key: UniqueKey(),
                      bottomLeftBadge: widget.flowChart.widget.itemLeftBottomBadges != null && widget.dashboard.elements[i].element != null ? 
                        widget.flowChart.widget.itemLeftBottomBadges!(widget.dashboard.elements[i].element as T) : null,
                      topRightBadge: widget.flowChart.widget.itemrightTopBadges != null && widget.dashboard.elements[i].element != null ? 
                        widget.flowChart.widget.itemrightTopBadges!(widget.dashboard.elements[i].element as T) : null,
                      dashboard: widget.dashboard,
                      element: widget.dashboard.elements.elementAt(i),
                      onElementPressed: widget.onElementPressed == null
                          ? null
                          : (context, position) => widget.onElementPressed!(
                                context,
                                position,
                                widget.dashboard.elements.elementAt(i),
                              ),
                      onElementSecondaryTapped: widget.onElementSecondaryTapped == null
                          ? null
                          : (context, position) => widget.onElementSecondaryTapped!(
                                context,
                                position,
                                widget.dashboard.elements.elementAt(i),
                              ),
                      onElementLongPressed: widget.onElementLongPressed == null
                          ? null
                          : (context, position) => widget.onElementLongPressed!(
                                context,
                                position,
                                widget.dashboard.elements.elementAt(i),
                              ),
                      onElementSecondaryLongTapped:
                          widget.onElementSecondaryLongTapped == null
                              ? null
                              : (context, position) =>
                                  widget.onElementSecondaryLongTapped!(
                                    context,
                                    position,
                                    widget.dashboard.elements.elementAt(i),
                                  ),
                      onHandlerPressed: widget.onHandlerPressed == null
                          ? null
                          : (context, position, handler, element) => widget
                              .onHandlerPressed!(context, position, handler, element),
                      onHandlerSecondaryTapped: widget.onHandlerSecondaryTapped == null
                          ? null
                          : (context, position, handler, element) =>
                              widget.onHandlerSecondaryTapped!(
                                context,
                                position,
                                handler,
                                element,
                              ),
                      onHandlerLongPressed: widget.onHandlerLongPressed == null
                          ? null
                          : (context, position, handler, element) =>
                              widget.onHandlerLongPressed!(
                                context,
                                position,
                                handler,
                                element,
                              ),
                      onHandlerSecondaryLongTapped:
                          widget.onHandlerSecondaryLongTapped == null
                              ? null
                              : (context, position, handler, element) =>
                                  widget.onHandlerSecondaryLongTapped!(
                                    context,
                                    position,
                                    handler,
                                    element,
                                  ),
                    ),
                  // Draw arrows        
                  for (int i = 0; i < widget.dashboard.elements.length; i++)
                    for (int n = 0; n < widget.dashboard.elements[i].next.length; n++) 
                      DrawArrow(
                        flow: this,
                        key: UniqueKey(),
                        index: n, 
                        srcElement: widget.dashboard.elements[i],
                        destElement: widget.dashboard.elements[widget.dashboard.findElementIndexById(
                          widget.dashboard.elements[i].next[n].destElementId,
                        )],
                        env: widget.dashboard.elements[i].next[n].env,
                        infos: widget.dashboard.elements[i].next[n].infos,
                        arrowParams: widget.dashboard.elements[i].next[n].arrowParams,
                        pivots: widget.dashboard.elements[i].next[n].pivots,
                      ),
                    // drawing segment handlers
                    for (int i = 0; i < widget.dashboard.elements.length; i++)
                      for (int n = 0; n < widget.dashboard.elements[i].next.length; n++)
                        if (widget.dashboard.elements[i].next[n].arrowParams.style == ArrowStyle.segmented)
                          for (int j = 0; j < widget.dashboard.elements[i].next[n].pivots.length; j++)
                            SegmentHandler(
                              key: UniqueKey(),
                              pivot: widget.dashboard.elements[i].next[n].pivots[j],
                              dashboard: widget.dashboard,
                              onPivotPressed: widget.onPivotPressed,
                              onPivotSecondaryPressed: widget.onPivotSecondaryPressed,
                            ),
                  // user drawing when connecting elements
                  DrawingArrowWidget(style: widget.dashboard.defaultArrowStyle),
                  Positioned(top: 0, right: 0, child: FlowChartMenu( 
                    key: widget.dashboard.chartMenuKey,
                    chart: this, 
                    menuExtension: widget.menuWidget != null ?  widget.menuWidget!(widget.dashboard) : null,
                    dashboard: widget.dashboard,
                    width: widget.flowChart.widget.width)
                ), 
              ],
    );
  }
}
