import 'package:flutter/material.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';

class FlowChartLeftMenu<T extends FlowData> extends StatefulWidget {
  Dashboard dashboard;
  List<String> categories;
  double height;
  double itemWidth;
  double innerMenuWidth;
  Widget Function()? menuExtension;
  TextEditingController ctrl = TextEditingController();
  final List<T> Function(String cat) draggableItemBuilder;

  List<Draggable<T>> Function(List<T> items) getDraggable;
  
  FlowChartLeftMenu ({ super.key, required this.categories, required this.dashboard, 
    this.height = 100,
    this.innerMenuWidth = 100,
    this.itemWidth = 100,
    this.menuExtension,
    required this.getDraggable,
    required this.draggableItemBuilder
  });
  @override FlowChartLeftMenuState createState() => FlowChartLeftMenuState();
}
class FlowChartLeftMenuState<T extends FlowData> extends State<FlowChartLeftMenu> {
  @override Widget build(BuildContext context) {
    List<Widget> menuItems = [];
    for (var cat in widget.categories) {
      if (widget.draggableItemBuilder(cat).isEmpty) continue;
      var items = widget.draggableItemBuilder(cat).where(
                (element) => element.getName().toLowerCase().contains(widget.ctrl.value.text.toLowerCase())).toSet().toList();
      menuItems.add(
        Container( width: widget.dashboard.isMenu ? widget.innerMenuWidth : 0, margin: const EdgeInsets.only(bottom: 0.3),
          decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: .5))),
          child: Stack( children: [
            widget.dashboard.isMenu && widget.innerMenuWidth < 200 ? Wrap( alignment: WrapAlignment.start, 
              children: widget.getDraggable(items))
          : Theme(
            data: Theme.of(context).copyWith(dividerColor: widget.dashboard.midDashColor),
                  child: ExpansionTile(
            collapsedShape: RoundedRectangleBorder(
                side: BorderSide.none,
            ),
            shape: RoundedRectangleBorder(
              side: BorderSide.none,
            ),
            initiallyExpanded: true,
            title: SizedBox( 
              child : Row( children: [
                Padding(padding: const EdgeInsets.only(right: 10), 
                  child: Icon(cat.toUpperCase().contains("DATA") ? Icons.grid_on : Icons.bookmark, color: Colors.grey)), 
                Flexible( 
                  child: Padding( 
                    padding: const EdgeInsets.only(right: 5), 
                    child: Text(cat.toUpperCase(), overflow: TextOverflow.ellipsis,
                    style: const TextStyle(color: Colors.black, fontSize: 11, fontWeight: FontWeight.w500)))) 
              ])
            ), 
            iconColor: Colors.white, 
            collapsedIconColor: Colors.white,
            children: [ 
              Container( width: widget.dashboard.isMenu ? widget.innerMenuWidth : 0, color: Colors.white,
                child : Wrap( alignment: WrapAlignment.center, 
                  children: widget.getDraggable(items))
              )],
            ))
          ]
        ))
      );
    }
    return Container( 
      height: widget.height,
      color: widget.dashboard.midDashColor,
      child: Stack( children: [ 
            Container(
                          width: widget.innerMenuWidth,
                          height: 50,
                          decoration: BoxDecoration(border: Border(
                            bottom: BorderSide(color: Colors.grey.shade400),
                            right: BorderSide(color: widget.dashboard.midDashColor),
                            left: BorderSide(color: Colors.white))),
                          child: TextFormField(
                            style: const TextStyle(color: Colors.black, fontSize: 15),
                            cursorColor: widget.dashboard.dashColor,
                            controller: widget.ctrl,
                            onChanged: (value) { setState(() { }); }, 
                            decoration: InputDecoration(
                              hintText: "search item...",
                              prefixIcon: const Icon(Icons.search, color: Colors.grey),
                              fillColor: widget.dashboard.midDashColor,
                              filled: true,
                              contentPadding: const EdgeInsets.only(left: 50, right: 50, top: 15, bottom: 15),
                              hintStyle: TextStyle(
                                color: Colors.grey,
                                fontSize: 15,
                                fontWeight: FontWeight.w400
                              ),
                              border: InputBorder.none
                            )
                          )
                        ),
            Container( 
              margin: EdgeInsets.only(top: 50),
              height: widget.height - 100,
              constraints: BoxConstraints(minWidth: widget.itemWidth),
              width: widget.dashboard.isMenu ? widget.innerMenuWidth : 0, 
              color: widget.dashboard.midDashColor,
              child: SingleChildScrollView( child: Column( children: [
                ...menuItems 
              ]) )
            ),
          Positioned( bottom: 0, height: 50,
              child: widget.menuExtension != null ? widget.menuExtension!() : Container(),),
      ])
    );
  }
}