import 'package:flutter/material.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:dotted_line/dotted_line.dart';

enum DisplayEnum {
  MENU,
  INFO
}

class FlowChartMenu extends StatefulWidget {
  ChartWidgetState chart;
  Dashboard dashboard;
  double width = 100;
  Widget? menuExtension;
  
  FlowChartMenu ({ super.key, required this.chart, required this.dashboard, this.width = 100, this.menuExtension });
  @override FlowChartMenuState createState() => FlowChartMenuState();
}
class FlowChartMenuState extends State<FlowChartMenu> {
  @override Widget build(BuildContext context) {
    bool isLow = widget.chart.widget.flowChart.widget.width < 600;
    GlobalKey<FormFieldState> zoomKey = GlobalKey<FormFieldState>();
    return Row( mainAxisAlignment: MainAxisAlignment.end, children : [ Container( // SHORTCUT
                    width: widget.width,
                    height: 50,
                    padding: const EdgeInsets.only(left: 20),
                    color: widget.dashboard.dashColor,
                    child: Row( children : [ Expanded(flex: 2, child: Row( children: [
                      widget.chart.widget.flowChart.widget.onDashboardAlertOpened == null ? Container() : Container( 
                        decoration: const BoxDecoration(
                          border: Border(right: BorderSide(color: Colors.white, width: 1))
                        ),
                        child: Row( children: [
                          Tooltip( message: "open file", child: Padding( padding: const EdgeInsets.only(right: 15), 
                            child: InkWell( mouseCursor: SystemMouseCursors.click,
                              onTap: () {
                                widget.dashboard.id = null;
                                widget.dashboard.name = "";
                                widget.dashboard.isOpened = true;
                                if (!widget.dashboard.inDialog) {
                                  widget.dashboard.inDialog = true;
                                  showDialog(
                                      barrierDismissible: false,
                                      context: context, builder: (context) {
                                      return AlertDialog(
                                        titlePadding: EdgeInsets.zero,
                                        insetPadding: EdgeInsets.zero,
                                        backgroundColor: Colors.white,
                                        shape: RoundedRectangleBorder( borderRadius: BorderRadius.circular(0)),
                                        title: widget.chart.widget.flowChart.widget.onDashboardAlertOpened!(
                                          context, widget.dashboard));
                                    });
                                }
                              }, 
                              child: Icon(Icons.folder, color: Colors.white)))),
                        ])),
                      InkWell( mouseCursor: SystemMouseCursors.click, child: Container( 
                        decoration: BoxDecoration(
                          border: Border(right: BorderSide(color: Colors.white, width: 1))
                        ),
                        child: Padding( padding: EdgeInsets.symmetric(horizontal: 10), 
                          child: PopupMenuButton<DisplayEnum>(
                            initialValue: null,
                            onSelected: (DisplayEnum value) {
                              if (value == DisplayEnum.MENU) {  widget.dashboard.isMenu = !widget.dashboard.isMenu; }
                              if (value == DisplayEnum.INFO) {  widget.dashboard.isInfo = !widget.dashboard.isInfo; }
                              widget.chart.widget.flowChart.setState(() {});
                            },
                            tooltip: "display",
                            itemBuilder: (BuildContext context) => <PopupMenuEntry<DisplayEnum>>[
                              PopupMenuItem<DisplayEnum>(
                                value: DisplayEnum.MENU,
                                child: Row( children: [ 
                                    Icon(widget.dashboard.isMenu ? Icons.remove_red_eye : Icons.remove_red_eye_outlined),
                                    Padding( padding: EdgeInsets.only(left: 10),
                                      child: Text(widget.dashboard.isMenu ? 'hide menu' : 'show menu', textAlign: TextAlign.center,))
                                ]),
                              ),
                              PopupMenuItem<DisplayEnum>(
                                value: DisplayEnum.INFO,
                                child: Row( children: [ 
                                    Icon(widget.dashboard.isMenu ? Icons.remove_red_eye : Icons.remove_red_eye_outlined),
                                    Padding( padding: EdgeInsets.only(left: 10),
                                      child: Text(widget.dashboard.isInfo ? 'hide info' : 'show info', textAlign: TextAlign.center,))
                                ]),
                              ),
                            ],
                            child: 
                            Row( children: [ Icon(Icons.fullscreen, color: Colors.white), Icon(Icons.arrow_drop_down, size: 10, color: Colors.white) ])
                            ),)
                        )
                      ),
                      InkWell( mouseCursor: SystemMouseCursors.click, child: Container( 
                        decoration: BoxDecoration(
                          border: Border(right: BorderSide(color: Colors.white, width: 1))
                        ),
                        child: Padding( padding: EdgeInsets.symmetric(horizontal: 10), 
                          child: PopupMenuButton<double>(
                            child: 
                            Row( children: [ Text("${(widget.dashboard.getZoomFactor() * 100).toInt()}% ", 
                            style: TextStyle(color: Colors.white),), Icon(Icons.arrow_drop_down, size: 10, color: Colors.white) ]),
                            initialValue: [0.25, .5, 75, 1, 2, 3, 4].contains(widget.dashboard.currentZoom) ? widget.dashboard.currentZoom : null,
                            onSelected: (double value) {
                              widget.dashboard.setZoomFactor(value);
                            },
                            tooltip: "custom zoom",
                            itemBuilder: (BuildContext context) => <PopupMenuEntry<double>>[
                              PopupMenuItem<double>(
                                enabled: false,
                                padding: EdgeInsets.all(0),
                                value: widget.dashboard.currentZoom,
                                child: Wrap( alignment: WrapAlignment.center, children: [ 
                                  Padding( padding: EdgeInsets.only(left: 10, top: 10, bottom: 10) ,  child: TextFormField(
                                  key: zoomKey,
                                  cursorColor: widget.dashboard.dashColor,
                                  onChanged: (value) { }, 
                                  validator: (value) {
                                    try { 
                                      double.parse(value ?? ""); 
                                      return null;
                                    } catch (e) {  return "Invalid number"; }
                                  },
                                  decoration: const InputDecoration(
                                    constraints: BoxConstraints(maxWidth: 100, minWidth: 50, minHeight: 50, maxHeight: 50),
                                    hintText: "zoom...",
                                    fillColor: Colors.white,
                                    contentPadding: EdgeInsets.only(left: 20, right: 20, top: 5, bottom: 10),
                                    filled: true,
                                    hintStyle: TextStyle(
                                      color: Colors.black,
                                      fontSize: 14,
                                      fontWeight: FontWeight.w300
                                    ),
                                    border: InputBorder.none,
                                    enabledBorder: InputBorder.none
                                  )
                                )),
                                Tooltip(
                                      message: 'apply zoom',
                                      child:InkWell( 
                                        mouseCursor: SystemMouseCursors.click,
                                        onTap: () {
                                          if (zoomKey.currentState != null && zoomKey.currentState!.validate()) {
                                            widget.dashboard.setZoomFactor(double.parse(zoomKey.currentState?.value) / 100);
                                          }
                                        },
                                        child: Container(
                                          margin: EdgeInsets.only(top: 10),
                                          width: 48,
                                          height: 48,
                                          color: Colors.black,
                                          child: Icon(Icons.check, color: Colors.white)
                                        )
                                      )
                                    )
                                  ],
                                ),
                              ),
                              const PopupMenuItem<double>(
                                value: 0.25,
                                child: Text('25%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 0.5,
                                child: Text('50%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 0.75,
                                child: Text('75%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 1,
                                child: Text('100%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 1.25,
                                child: Text('125%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 1.50,
                                child: Text('150%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 2,
                                child: Text('200%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 3,
                                child: Text('300%'),
                              ),
                              const PopupMenuItem<double>(
                                value: 4,
                                child: Text('400%'),
                              ),
                              
                            ],
                          )
                        ),)),
                      isLow ? Container() : Container( 
                        decoration: BoxDecoration(
                          border: Border(right: BorderSide(color: Colors.white, width: 1))
                        ),
                        child: Row( children: [
                          Tooltip( message: "zoom in", child: Container( child: Padding( padding: EdgeInsets.symmetric(horizontal: 10), 
                            child: InkWell( mouseCursor: SystemMouseCursors.click, 
                            onTap: () {
                              widget.dashboard.setZoomFactor(widget.dashboard.zoomFactor + 0.1);  
                            },
                            child: Icon(Icons.zoom_in, color: Colors.white))))),
                          Tooltip( message: "zoom out", child:Container( child: Padding( padding: EdgeInsets.symmetric(horizontal: 10), 
                            child: InkWell( mouseCursor: SystemMouseCursors.click,
                              onTap: () {
                                widget.dashboard.setZoomFactor(widget.dashboard.zoomFactor - 0.1);  
                              }, 
                              child: Icon(Icons.zoom_out, color: Colors.white))))),
                        ])),
                      Container( 
                        decoration: BoxDecoration(
                          border: Border(right: BorderSide(color: Colors.white, width: 1))
                        ),
                        child: Row( children: [
                          Padding( padding: EdgeInsets.symmetric(horizontal: 10), 
                            child: InkWell( 
                              onTap: () {
                                if (widget.dashboard.canBack()) {
                                  widget.dashboard.back();
                                }
                              },
                              mouseCursor: !widget.dashboard.canBack() ? MouseCursor.defer : SystemMouseCursors.click, 
                              child: Icon(Icons.undo, color: !widget.dashboard.canBack() ? widget.dashboard.midDashColor : Colors.white)),),
                          Padding( padding: EdgeInsets.symmetric(horizontal: 10), 
                            child: InkWell( 
                              onTap: () {
                                if (widget.dashboard.canForward()) {
                                  widget.dashboard.forward();
                                }
                              },
                              mouseCursor: !widget.dashboard.canForward() ? MouseCursor.defer : SystemMouseCursors.click, child: Icon(Icons.redo, 
                              color: !widget.dashboard.canForward() ? widget.dashboard.midDashColor : Colors.white))),
                          
                        ])),
                        Expanded( 
                           child: Padding( padding: const EdgeInsets.symmetric(horizontal: 20), 
                           child: Text("current workflow : ${widget.dashboard.name}", overflow: TextOverflow.ellipsis,
                            style: const TextStyle(color: Colors.white, fontSize: 14), textAlign: TextAlign.start))),
                      ])),
                      widget.menuExtension != null && widget.chart.widget.flowChart.widget.width > 600 ? Container( 
                        decoration: const BoxDecoration(
                          border: Border(left: BorderSide(color: Colors.white, width: 1))
                        ),
                        child:  widget.menuExtension
                      ) : Container()
                    ])
                  ) ]);
  }
}

class MySeparator extends StatelessWidget {
  double width = 1; double dashWidth = 10; double dashSpace = 10;
  MySeparator({Key? key, this.width = 1,  this.dashSpace = 10, this.dashWidth = 10,
    this.height = 1, this.color = Colors.black})
      : super(key: key);
  final double height;
  final Color color;

  @override
  Widget build(BuildContext context) {
    return Container( width: width, child: dashSpace == 0 ? 
      Divider( thickness: 2, color: color ) 
      : DottedLine(
              dashLength: dashWidth,
              dashGapLength: dashSpace,
              lineThickness: 2,
              dashColor: color
            )
          );
  }
}