import 'package:flutter/material.dart';
import 'package:dotted_line/dotted_line.dart';
import 'package:flutter_flow_chart/src/ui/draw_arrow.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:flutter_colorpicker/flutter_colorpicker.dart';
import 'package:number_text_input_formatter/number_text_input_formatter.dart';
import 'package:uuid/uuid.dart';

class FlowChartSelectedMenu extends StatefulWidget {
  Dashboard dashboard;
  double height = 100;
  bool isDashboardInfo = true;
  FlowChartSelectedMenu ({ super.key, required this.dashboard, this.height = 100 });
  @override FlowChartSelectedMenuState createState() => FlowChartSelectedMenuState();
}
class FlowChartSelectedMenuState extends State<FlowChartSelectedMenu> {
  @override Widget build(BuildContext context) {
    Widget? w;
    if (widget.isDashboardInfo && widget.dashboard.elementSelected.length == 1) {
      w = Container(
        width: 200,
        height: widget.height,
        color: widget.dashboard.midDashColor,
        child: SingleChildScrollView( child: Column( children: [ ...widget.dashboard.infoItemWidget != null ?
          widget.dashboard.infoItemWidget!(widget.dashboard.elementSelected.first.element, widget.dashboard.elementSelected.first.id) : [],
          widget.dashboard.arrowsSelected.isNotEmpty || widget.dashboard.elementSelected.isNotEmpty ? Container( 
                          width: 200,
                          margin: EdgeInsets.only(top: 15),
                          decoration: BoxDecoration(border: Border(
                            top: BorderSide(color: Colors.grey, width: 1))),
                          child: Column(crossAxisAlignment: CrossAxisAlignment.center, children: [
                            Tooltip( message: "remove",
                              child: InkWell( mouseCursor: SystemMouseCursors.click, 
                              onTap: () { 
                                widget.dashboard.removeElements((element) {
                                  if (element.isSelected) {
                                    widget.dashboard.arrows.removeWhere( (e) => e.toID.contains(element.id) || e.fromID.contains(element.id));
                                  }
                                  return element.isSelected;
                                }, context); 
                                Future.delayed(Duration(milliseconds: 100), () {
                                  widget.dashboard.flutterChartKey.currentState?.setState(() { });
                                });
                              }, child: Container( margin: EdgeInsets.all(10),
                                decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), border: Border.all(color: Colors.black, width: 1)),
                                width: 200, height: 30,
                                child: Icon(Icons.delete_outline, color: Colors.black),
                              ))
                            ),
                            Tooltip( message: "copy",
                              child: InkWell( mouseCursor: SystemMouseCursors.click, 
                              onTap: () {
                                for (var sel in widget.dashboard.elementSelected) {
                                  var el =FlowElement.fromMap(widget.dashboard, sel.toMap());
                                  el.id = Uuid().v8();
                                  widget.dashboard.addElement(FlowElement.fromMap(widget.dashboard, sel.toMap()), context);
                                  widget.dashboard.elements.last.position += Offset(50, 50);
                                }
                                Future.delayed(Duration(milliseconds: 100), () {
                                  widget.dashboard.chartKey.currentState?.setState(() { });
                                });
                              }, child: Container( margin: EdgeInsets.only(left: 10, right: 10, bottom: 10),
                                decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), border: Border.all(color: Colors.black, width: 1)),
                                width: 200, height: 30,
                                child: Icon(Icons.copy, color: Colors.black),
                              ))
                            ),
                          ])
                        ) : Container()
        ])
      ));
    } else if (widget.isDashboardInfo && widget.dashboard.arrowsSelected.length == 1) {
      w = Container(
        width: 200,
        height: widget.height,
        color: widget.dashboard.midDashColor,
        child: SingleChildScrollView( child: Column( children: [ 
          widget.dashboard.infoItemWidget != null ?
          widget.dashboard.infoLinkWidget!(widget.dashboard.arrowsSelected.first) : Container(),
          widget.dashboard.arrowsSelected.isNotEmpty || widget.dashboard.elementSelected.isNotEmpty ? Container( 
                          width: 200,
                          margin: const EdgeInsets.only(top: 15),
                          decoration: const BoxDecoration(border: Border(top: BorderSide(color: Colors.grey, width: 1))),
                          child: Column(crossAxisAlignment: CrossAxisAlignment.center, children: [
                            Tooltip( message: "remove",
                              child: InkWell( mouseCursor: SystemMouseCursors.click, 
                              onTap: () {
                                widget.dashboard.removeArrows((element) { 
                                  if (element.isSelected && element.elementIndex != null && element.connIndex != null) {
                                    widget.dashboard.elements[element.elementIndex!].next.removeAt(element.connIndex!);
                                  }
                                  return element.isSelected;
                                }, context); 
                                Future.delayed( const Duration(milliseconds: 100), () {
                                  widget.dashboard.flutterChartKey.currentState?.setState(() { });
                                });
                              }, child: Container( margin: const EdgeInsets.all(10),
                                decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), border: Border.all(color: Colors.black, width: 1)),
                                width: 200, height: 30,
                                child: const Icon(Icons.delete_outline, color: Colors.black),
                              ))
                            ),
                          ])
                        ) : Container()
        ])
      ));
    } else if (widget.isDashboardInfo && widget.dashboard.infoWidget != null) {
       w = Container(
        width: 200,
        height: widget.height,
        color: widget.dashboard.midDashColor,
        child: Column( children: widget.dashboard.infoWidget != null ? widget.dashboard.infoWidget!() : [])
       );
    } else {
      w = Container( // SHORTCUT
                    width: 200,
                    height: widget.height,
                    color: widget.dashboard.midDashColor,
                    child: Column( children: [ 
                      Container( padding: const EdgeInsets.all(10), width: 200, height: 60,
                        decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1))),
                        child: Column( mainAxisAlignment: MainAxisAlignment.center, children: [
                          Text("STYLE ${widget.dashboard.elementSelected.isNotEmpty ? "ELEMENT" : "ARROW"}", style: TextStyle(fontSize: 15, fontWeight: FontWeight.bold), textAlign: TextAlign.center),
                          Text("<${widget.dashboard.arrowsSelected.isEmpty && widget.dashboard.elementSelected.isEmpty ? "general" : "selected"}>", style: TextStyle(fontSize: 12), textAlign: TextAlign.center),
                        ])),
                      SizedBox( width: 200, height: widget.height - 60, child: SingleChildScrollView( child: Column( children: [
                      widget.dashboard.elementSelected.isNotEmpty ? Container() : Container( padding: EdgeInsets.symmetric(horizontal: 10, vertical: 20), 
                        decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1))),
                        child: Column( children: [
                          Row( children: [
                          InkWell( mouseCursor: SystemMouseCursors.click, 
                          child: Padding( padding: const EdgeInsets.only(left: 10, right: 10), 
                            child: PopupMenuButton<ArrowDash>(
                              tooltip: "line defaults",
                              constraints: const BoxConstraints(maxWidth: 100),
                              initialValue: null,
                              onSelected: (ArrowDash value) {
                                if (widget.dashboard.elementSelected.isEmpty) {
                                  for(var sel in widget.dashboard.arrowsSelected) { 
                                    sel.params.dashSpace = spaceArrowDash(value); 
                                    sel.params.dashWidth = widthArrowDash(value);
                                  }
                                  widget.dashboard.chartKey.currentState?.setState(() { });
                                }
                                widget.dashboard.defaultDashSpace = spaceArrowDash(value);
                                widget.dashboard.defaultDashWidth = widthArrowDash(value);
                                setState(() {});
                              },
                              itemBuilder: (BuildContext context) => <PopupMenuEntry<ArrowDash>>[
                                PopupMenuItem<ArrowDash>(
                                  value: ArrowDash.line,
                                  child: Row( mainAxisAlignment: MainAxisAlignment.center, children: [ 
                                      MySeparator(width: 25, dashWidth: widthArrowDash(ArrowDash.line), 
                                      dashSpace: spaceArrowDash(ArrowDash.line),)
                                  ]),
                                ),
                                PopupMenuItem<ArrowDash>(
                                  value: ArrowDash.largeDash,
                                  child: Row( mainAxisAlignment: MainAxisAlignment.center, children: [ 
                                    MySeparator(width: 25, dashWidth: widthArrowDash(ArrowDash.largeDash), dashSpace: spaceArrowDash(ArrowDash.largeDash),)
                                  ]),
                                ),
                                PopupMenuItem<ArrowDash>(
                                  value: ArrowDash.mediumDash,
                                  child: Row( mainAxisAlignment: MainAxisAlignment.center, children: [ 
                                    MySeparator(width: 25, dashWidth: widthArrowDash(ArrowDash.mediumDash), dashSpace: spaceArrowDash(ArrowDash.mediumDash),)
                                  ]),
                                ),
                                PopupMenuItem<ArrowDash>(
                                  value: ArrowDash.smallDash,
                                  child: Row( mainAxisAlignment: MainAxisAlignment.center, children: [ 
                                    MySeparator(width: 25, dashWidth: widthArrowDash(ArrowDash.smallDash), dashSpace: spaceArrowDash(ArrowDash.smallDash),)
                                  ]),
                                ),
                                PopupMenuItem<ArrowDash>(
                                  value: ArrowDash.heavyDotted,
                                  child: Row( mainAxisAlignment: MainAxisAlignment.center, children: [ 
                                    MySeparator(width: 25, dashWidth: widthArrowDash(ArrowDash.heavyDotted), dashSpace: spaceArrowDash(ArrowDash.heavyDotted),)
                                  ]),
                                ),
                                PopupMenuItem<ArrowDash>(
                                  value: ArrowDash.mediumDotted,
                                  child: Row( mainAxisAlignment: MainAxisAlignment.center, children: [ 
                                    MySeparator(width: 25, dashWidth: widthArrowDash(ArrowDash.mediumDotted), dashSpace: spaceArrowDash(ArrowDash.mediumDotted),)
                                  ]),
                                ),
                                PopupMenuItem<ArrowDash>(
                                  value: ArrowDash.lightDotted,
                                  child: Row( mainAxisAlignment: MainAxisAlignment.center, children: [ 
                                    MySeparator(width: 25, dashWidth: widthArrowDash(ArrowDash.lightDotted), dashSpace: spaceArrowDash(ArrowDash.lightDotted),)
                                  ]),
                                ),
                              ],
                              child: Row( children: [ 
                                MySeparator(
                                  width: 35, 
                                  dashWidth: widget.dashboard.defaultDashWidth, 
                                  dashSpace: widget.dashboard.defaultDashSpace, 
                                  color: Colors.black 
                                ),
                                const SizedBox(height: 25, width: 10),
                                const Icon(Icons.arrow_drop_down, size: 10, color: Colors.black) ])
                              ),
                            )
                          ),
                          PopupMenuButton<void>(
                                tooltip: "color picker",
                                constraints: const BoxConstraints(maxWidth: 664),
                                initialValue: null,
                                onSelected: (void value) {},
                                itemBuilder: (BuildContext context) => <PopupMenuEntry<void>>[
                                  PopupMenuItem<void>(
                                    child: ColorPicker(pickerColor: Colors.black, onColorChanged: (value) {
                                      if (widget.dashboard.elementSelected.isEmpty) {
                                        for(var sel in widget.dashboard.arrowsSelected) { sel.params.color = value; }
                                        widget.dashboard.chartKey.currentState?.setState(() { });
                                      }
                                      setState(() { widget.dashboard.defaultColor = value; });
                                    },),
                                  ),
                                ],
                                child: Row( children: [ 
                                  Container(width: 15, height: 15, color: widget.dashboard.defaultColor),
                                  const SizedBox(height: 25, width: 5),
                                  const Icon(Icons.arrow_drop_down, size: 10, color: Colors.black) ])
                          ),
                          Tooltip( message: "stroke width",
                            child: Container(
                              margin: const EdgeInsets.only(left: 10),
                              width: 55, height: 25,
                              child: TextFormField( textAlign: TextAlign.center,
                              readOnly: widget.dashboard.defaultDashWidth <= 0,
                              initialValue: "${widget.dashboard.defaultStroke}",
                              onChanged: (value) { 
                                if (widget.dashboard.elementSelected.isEmpty) {
                                  for(var sel in widget.dashboard.arrowsSelected) { sel.params.thickness = double.parse(value); }
                                  widget.dashboard.chartKey.currentState?.setState(() { });
                                }
                                setState(() { widget.dashboard.defaultStroke = double.parse(value); });
                              },
                              style: const TextStyle(fontSize: 12),
                              decoration: const InputDecoration(
                                fillColor: Colors.white,
                                filled: true,
                                labelText: "stroke",
                                labelStyle: TextStyle(fontSize: 10),
                                border: OutlineInputBorder(),
                                contentPadding: EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                              ),
                              inputFormatters: [
                                NumberTextInputFormatter(
                                  integerDigits: 100,
                                  decimalDigits: 1,
                                  maxValue: '99',
                                  decimalSeparator: '.',
                                  groupSeparator: ',',
                                  allowNegative: false,
                                  overrideDecimalPoint: false,
                                  insertDecimalPoint: false,
                                  insertDecimalDigits: false,
                                ),
                              ],
                              keyboardType: TextInputType.number,
                          )))
                        ]),
                        Row(children: [
                          InkWell( mouseCursor: SystemMouseCursors.click, child: Padding( 
                            padding: const EdgeInsets.only(left: 10, top: 10, right: 10), 
                            child: PopupMenuButton<ArrowStyle>(
                              initialValue: null,
                              onSelected: (ArrowStyle value) {
                                if (widget.dashboard.elementSelected.isEmpty) {
                                  for(var sel in widget.dashboard.arrowsSelected) { sel.params.style = value; }
                                  widget.dashboard.chartKey.currentState?.setState(() { });
                                }
                                widget.dashboard.defaultArrowStyle = value;
                                setState(() {});
                              },
                              tooltip: "line styles",
                              itemBuilder: (BuildContext context) => <PopupMenuEntry<ArrowStyle>>[
                                const PopupMenuItem<ArrowStyle>(
                                  value: ArrowStyle.segmented,
                                  child: Row( children: [ 
                                      Icon(Icons.turn_slight_left),
                                      Padding( padding: EdgeInsets.only(left: 10),
                                        child: Text('straight', textAlign: TextAlign.center,))
                                  ]),
                                ),
                                const PopupMenuItem<ArrowStyle>(
                                  value: ArrowStyle.curve,
                                  child: Row( children: [ 
                                      Icon(Icons.roundabout_left),
                                      Padding( padding: EdgeInsets.only(left: 10),
                                        child: Text('curved', textAlign: TextAlign.center,))
                                  ]),
                                ),
                                const PopupMenuItem<ArrowStyle>(
                                  value: ArrowStyle.rectangular,
                                  child: Row( children: [ 
                                      Icon(Icons.turn_sharp_left_outlined),
                                      Padding( padding: EdgeInsets.only(left: 10),
                                        child: Text('rectangular', textAlign: TextAlign.center,))
                                  ]),
                                ),
                              ],
                              child: 
                              Row( children: [ Icon(widget.dashboard.defaultArrowStyle == ArrowStyle.segmented ? Icons.turn_slight_left : widget.dashboard.defaultArrowStyle == ArrowStyle.curve ? Icons.roundabout_left : Icons.turn_sharp_left_outlined
                                , color: Colors.black), 
                              const Icon(Icons.arrow_drop_down, size: 10, color: Colors.black) ])
                              )
                            )
                          ),
                          Tooltip( message: "space dash",
                            child: Container(
                              margin: const EdgeInsets.only(top: 10),
                              width: 105 / 2, height: 25,
                              child: TextFormField( textAlign: TextAlign.center,
                              readOnly: widget.dashboard.defaultDashWidth <= 0,
                              initialValue: "${widget.dashboard.defaultDashWidth}",
                              onChanged: (value) { 
                                if (widget.dashboard.elementSelected.isEmpty) {
                                  for(var sel in widget.dashboard.arrowsSelected) { sel.params.dashWidth = double.parse(value); }
                                  widget.dashboard.chartKey.currentState?.setState(() { });
                                }
                                setState(() { widget.dashboard.defaultDashWidth = double.parse(value); });
                              },
                              style: const TextStyle(fontSize: 12),
                              decoration: InputDecoration(
                                fillColor: widget.dashboard.defaultDashWidth <= 0 ? widget.dashboard.midDashColor : Colors.white,
                                filled: true,
                                labelText: "dash",
                                labelStyle: const TextStyle(fontSize: 10),
                                border: const OutlineInputBorder(),
                                contentPadding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                              ),
                              inputFormatters: [
                                NumberTextInputFormatter(
                                  integerDigits: 100,
                                  decimalDigits: 0,
                                  maxValue: '99',
                                  decimalSeparator: '.',
                                  groupSeparator: ',',
                                  allowNegative: false,
                                  overrideDecimalPoint: false,
                                  insertDecimalPoint: false,
                                  insertDecimalDigits: false,
                                ),
                              ],
                              keyboardType: TextInputType.number,
                          ))),
                          Tooltip( message: "space width",
                            child: Container(
                              margin: const EdgeInsets.only(left: 10, top: 10),
                              width: 105 / 2, height: 25,
                              child: TextFormField( textAlign: TextAlign.center,
                              initialValue: "${widget.dashboard.defaultDashSpace}",
                              onChanged: (value) { 
                                if (widget.dashboard.elementSelected.isEmpty) {
                                  for(var sel in widget.dashboard.arrowsSelected) { sel.params.dashSpace = double.parse(value); }
                                  widget.dashboard.chartKey.currentState?.setState(() { });
                                }
                                setState(() { widget.dashboard.defaultDashSpace = double.parse(value); });
                              },
                              style: const TextStyle(fontSize: 12),
                              decoration: InputDecoration(
                                fillColor: widget.dashboard.defaultDashWidth <= 0 ? widget.dashboard.midDashColor : Colors.white,
                                filled: true,
                                labelText: "space",
                                labelStyle: const TextStyle(fontSize: 10),
                                border: const OutlineInputBorder(),
                                contentPadding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                              ),
                              inputFormatters: [
                                NumberTextInputFormatter(
                                  integerDigits: 100,
                                  decimalDigits: 0,
                                  maxValue: '99',
                                  decimalSeparator: '.',
                                  groupSeparator: ',',
                                  allowNegative: false,
                                  overrideDecimalPoint: false,
                                  insertDecimalPoint: false,
                                  insertDecimalDigits: false,
                                ),
                              ],
                              keyboardType: TextInputType.number,
                          )))
                        ]), 
                      ])),
                      widget.dashboard.elementSelected.isNotEmpty ? Container() : 
                      Container( padding: const EdgeInsets.only(left: 10, right: 10, bottom: 20, top: 15), 
                        decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1))),
                        child: Column( children: [
                          Row( mainAxisAlignment: MainAxisAlignment.center, children : [ 
                            InkWell( mouseCursor: SystemMouseCursors.click, 
                            child: Padding( padding: const EdgeInsets.symmetric(horizontal: 10), 
                              child: PopupMenuButton<ArrowDirection>(
                                initialValue: null,
                                onSelected: (ArrowDirection value) {
                                  if (widget.dashboard.elementSelected.isEmpty) {
                                    for(var sel in widget.dashboard.arrowsSelected) { sel.params.direction = value; }
                                    widget.dashboard.chartKey.currentState?.setState(() { });
                                  }
                                  widget.dashboard.defaultArrowDirection = value;
                                  setState(() {});
                                },
                                tooltip: widget.dashboard.defaultArrowDirection == ArrowDirection.forward ? 'forward' : widget.dashboard.defaultArrowDirection == ArrowDirection.backward ? 'backward' : 'bidirectionnal',
                                itemBuilder: (BuildContext context) => <PopupMenuEntry<ArrowDirection>>[
                                  const PopupMenuItem<ArrowDirection>(
                                    value: ArrowDirection.forward,
                                    child: Row( children: [ 
                                        Icon(Icons.arrow_forward),
                                        Padding( padding: EdgeInsets.only(left: 10),
                                          child: Text('forward', textAlign: TextAlign.center,))
                                    ]),
                                  ),
                                  const PopupMenuItem<ArrowDirection>(
                                    value: ArrowDirection.backward,
                                    child: Row( children: [ 
                                        Icon(Icons.arrow_back),
                                        Padding( padding: EdgeInsets.only(left: 10),
                                          child: Text('curved', textAlign: TextAlign.center,))
                                    ]),
                                  ),
                                  const PopupMenuItem<ArrowDirection>(
                                    value: ArrowDirection.bidirectionnal,
                                    child: Row( children: [ 
                                        Icon(Icons.sync_alt_outlined),
                                        Padding( padding: EdgeInsets.only(left: 10),
                                          child: Text('bidirectionnal', textAlign: TextAlign.center,))
                                    ]),
                                  ),
                                ],
                                child: 
                                Row( children: [ 
                                  Icon(widget.dashboard.defaultArrowDirection == ArrowDirection.forward ? Icons.arrow_forward : widget.dashboard.defaultArrowDirection == ArrowDirection.backward ? Icons.arrow_back : Icons.sync_alt_outlined, color: Colors.black), 
                                  Padding( padding: const EdgeInsets.symmetric(horizontal: 10), 
                                    child: Text(widget.dashboard.defaultArrowDirection == ArrowDirection.forward ? 'forward' : widget.dashboard.defaultArrowDirection == ArrowDirection.backward ? 'backward' : 'bidirectionnal')),
                                  const Icon(Icons.arrow_drop_down, size: 10, color: Colors.black) ])
                                ),)
                            ),
                          ]),
                          Row(children: [
                            Tooltip( message: "forward size",
                              child: Container(
                                margin: const EdgeInsets.only(top: 10),
                                width: 150, height: 25,
                                child: TextFormField( textAlign: TextAlign.center,
                                initialValue: "${widget.dashboard.defaultForwardWidth}",
                                onChanged: (value) { 
                                  if (widget.dashboard.elementSelected.isEmpty) {
                                    for(var sel in widget.dashboard.arrowsSelected) { 
                                      try {
                                        sel.params.forwardWidth = double.parse(value); 
                                      } catch(e) {
                                        sel.params.forwardWidth = 0; 
                                      }
                                    }
                                    widget.dashboard.chartKey.currentState?.setState(() { });
                                  }
                                  setState(() { 
                                    try {
                                      widget.dashboard.defaultForwardWidth = double.parse(value); 
                                    } catch(e) {
                                      widget.dashboard.defaultForwardWidth = 0; 
                                    }
                                  });
                                },
                                style: const TextStyle(fontSize: 12),
                                decoration: InputDecoration(
                                  fillColor: widget.dashboard.defaultDashWidth <= 0 ? widget.dashboard.midDashColor : Colors.white,
                                  filled: true,
                                  labelText: "forward size",
                                  labelStyle: const TextStyle(fontSize: 10),
                                  border: const OutlineInputBorder(),
                                  contentPadding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                                ),
                                inputFormatters: [
                                  NumberTextInputFormatter(
                                    integerDigits: 50,
                                    decimalDigits: 0,
                                    maxValue: '50',
                                    decimalSeparator: '.',
                                    groupSeparator: ',',
                                    allowNegative: false,
                                    overrideDecimalPoint: false,
                                    insertDecimalPoint: false,
                                    insertDecimalDigits: false,
                                  ),
                                ],
                                keyboardType: TextInputType.number,
                            ))),
                            const Padding( padding: EdgeInsets.only(top: 10, left: 5), child: Icon(Icons.arrow_forward, color: Colors.black))
                          ]),
                          Row(children: [
                            Tooltip( message: "back size",
                              child: Container(
                                margin: const EdgeInsets.only(top: 10),
                                width: 150, height: 25,
                                child: TextFormField( textAlign: TextAlign.center,
                                initialValue: "${widget.dashboard.defaultBackWidth}",
                                onChanged: (value) { 
                                  if (widget.dashboard.elementSelected.isEmpty) {
                                    for(var sel in widget.dashboard.arrowsSelected) {
                                      try {
                                        sel.params.backwardWidth = double.parse(value); 
                                      } catch(e) {
                                        sel.params.backwardWidth = 0; 
                                      }
                                    }
                                    widget.dashboard.chartKey.currentState?.setState(() { });
                                  }
                                  setState(() { 
                                    try {
                                      widget.dashboard.defaultBackWidth = double.parse(value); 
                                    } catch(e) {
                                      widget.dashboard.defaultBackWidth = 0; 
                                    }
                                  });
                                },
                                style: const TextStyle(fontSize: 12),
                                decoration: InputDecoration(
                                  fillColor: widget.dashboard.defaultDashWidth <= 0 ? widget.dashboard.midDashColor : Colors.white,
                                  filled: true,
                                  labelText: "back size",
                                  labelStyle: const TextStyle(fontSize: 10),
                                  border: const OutlineInputBorder(),
                                  contentPadding: const EdgeInsets.symmetric(horizontal: 10, vertical: 5),
                                ),
                                inputFormatters: [
                                  NumberTextInputFormatter(
                                    integerDigits: 50,
                                    decimalDigits: 0,
                                    maxValue: '50',
                                    decimalSeparator: '.',
                                    groupSeparator: ',',
                                    allowNegative: false,
                                    overrideDecimalPoint: false,
                                    insertDecimalPoint: false,
                                    insertDecimalDigits: false,
                                  ),
                                ],
                                keyboardType: TextInputType.number,
                            ))),
                            const Padding( padding: EdgeInsets.only(top: 10, left: 5), child: Icon(Icons.arrow_back, color: Colors.black))
                          ])
                        ])),
                        widget.dashboard.arrowsSelected.isNotEmpty || widget.dashboard.elementSelected.isNotEmpty ? Container( 
                          width: 200,
                          decoration: const BoxDecoration(border: Border(bottom: BorderSide(color: Colors.grey, width: 1))),
                          child: Column(crossAxisAlignment: CrossAxisAlignment.center, children: [
                            Tooltip( message: "remove",
                              child: InkWell( mouseCursor: SystemMouseCursors.click, 
                              onTap: () {
                                widget.dashboard.removeElements((element) {
                                  if (element.isSelected) {
                                    widget.dashboard.arrows.removeWhere( (e) => e.toID.contains(element.id) || e.fromID.contains(element.id));
                                  }
                                  return element.isSelected;
                                }, context);
                                Future.delayed(const Duration(milliseconds: 100), () {
                                  widget.dashboard.flutterChartKey.currentState?.setState(() { });
                                });
                              }, child: Container( margin: const EdgeInsets.all(10),
                                decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), border: Border.all(color: Colors.black, width: 1)),
                                width: 200, height: 30,
                                child: const Icon(Icons.delete_outline, color: Colors.black),
                              ))
                            ),
                            Tooltip( message: "copy",
                              child: InkWell( mouseCursor: SystemMouseCursors.click, 
                              onTap: () {
                                for (var sel in widget.dashboard.elementSelected) {
                                  widget.dashboard.addElement(FlowElement.fromMap(widget.dashboard, sel.toMap()), context);
                                  widget.dashboard.elements.last.position += const Offset(50, 50);
                                }
                                Future.delayed( const Duration(milliseconds: 100), () {
                                  widget.dashboard.chartKey.currentState?.setState(() { });
                                });
                              }, child: Container( margin: const EdgeInsets.only(left: 10, right: 10, bottom: 10),
                                decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), border: Border.all(color: Colors.black, width: 1)),
                                width: 200, height: 30,
                                child: const Icon(Icons.copy, color: Colors.black),
                              ))
                            ),
                          ])
                        ) : Container()
                      ])))
                    ])
                  );
    }
    return Column( children: [
      Container( // SHORTCUT
        width: 200, height: 50,
        decoration: BoxDecoration(color: widget.dashboard.midDashColor, 
          border: const Border(bottom: BorderSide(color: Colors.grey, width: 1))),
        child: Row( children: [ 
          Tooltip( message: "dashboard information",
            child: InkWell( onTap: () => setState(() {widget.isDashboardInfo = true; }),
              mouseCursor: SystemMouseCursors.click,
              child: Container( alignment: Alignment.center, 
                padding: const EdgeInsets.symmetric(vertical: 10),
                color: widget.isDashboardInfo ? Colors.grey : widget.dashboard.midDashColor,
                width: 200 / 2, child: const Icon(Icons.info, color: Colors.white))
              )
            ),
            Tooltip( 
              message: "element style",
              child: InkWell( onTap: () => setState(() {widget.isDashboardInfo = false; }),
                mouseCursor: SystemMouseCursors.click,
                child: Container( alignment: Alignment.center, 
                  padding: const EdgeInsets.symmetric(vertical: 10),
                  color: !widget.isDashboardInfo ? Colors.grey : widget.dashboard.midDashColor,
                  width: 200 / 2, child: const Icon(Icons.format_paint, color: Colors.white)), 
                )
              )
          ]
        )
      ),
      widget.dashboard.error != null ? Container( width: 200, color: Colors.red, padding: EdgeInsets.all(10),
        child: Center( child: Text(widget.dashboard.error!, 
        style: const TextStyle(color: Colors.white, fontSize: 15), textAlign: TextAlign.center))) : Container(),
      w
    ]);
  }
}

// ignore: must_be_immutable
class MySeparator extends StatelessWidget {
  double width = 1; double dashWidth = 10; double dashSpace = 10;
  MySeparator({super.key, this.width = 1,  this.dashSpace = 10, this.dashWidth = 10,
    this.height = 1, this.color = Colors.black});
  final double height;
  final Color color;

  @override
  Widget build(BuildContext context) {
    return SizedBox( width: width, child: dashSpace == 0 ? 
      Divider( thickness: 2, color: color ) 
      : DottedLine(
              dashLength: dashWidth,
              dashGapLength: dashSpace,
              lineThickness: 2,
              dashColor: color
            )
          );
  }
}