import 'package:flutter/material.dart';
import 'package:flutter_flow_chart/flutter_flow_chart.dart';
import 'package:flutter_flow_chart/src/ui/element_widget.dart';
import 'package:flutter_flow_chart/src/ui/handler_widget.dart';

/// The widget to press and drag to resize the element
class ResizeWidget extends StatefulWidget {
  Widget? bottomLeftBadge;
  Widget? bottomRightBadge;
  Widget? topLeftBadge;
  Widget? topRightBadge;
  ///
  ResizeWidget({
    required this.comp,
    required this.element,
    required this.dashboard,
    required this.child,
    this.bottomLeftBadge,
    this.bottomRightBadge,
    this.topLeftBadge,
    this.topRightBadge,
    this.handlerColor =const Color.fromRGBO(38, 166, 154, 1),
    super.key,
  });

  Color handlerColor = Color.fromRGBO(38, 166, 154, 1); 
  final ElementWidgetState comp;
  ///
  final Dashboard dashboard;

  ///
  final FlowElement element;

  ///
  final Widget child;
  @override
  State<ResizeWidget> createState() => _ResizeWidgetState();
}

class _ResizeWidgetState extends State<ResizeWidget> {
  late Size elementStartSize;
  late Offset elementStartPosition;
  @override
  Widget build(BuildContext context) {
    return Container(
      width: widget.element.size.width,
      height: widget.element.size.height,
      child: Stack(
        children: [
          widget.child,
          widget.topLeftBadge == null ? Container() : Positioned(top: 0, left: 0, child: widget.topLeftBadge!),
          widget.topRightBadge == null ? Container() : Positioned(top: 0, right: 0, child: widget.topRightBadge!),
          widget.bottomRightBadge == null ? Container() : Positioned(bottom: 0, right: 0, child: widget.bottomRightBadge!),
          widget.bottomLeftBadge == null ? Container() : Positioned(bottom: 0, left: 0, child: widget.bottomLeftBadge!),
          _handler(Alignment.topLeft),
          _handler(Alignment.topRight),
          _handler(Alignment.bottomLeft),
          _handler(Alignment.bottomRight),
        ],
      ),
    );
  }

  Widget _handler(Alignment alignment) {
    return Listener(
      onPointerDown: (event) {
        elementStartSize = widget.element.size;
      },
      onPointerMove: (event) {
        if (alignment == Alignment.topLeft) {
          elementStartSize += -event.localDelta;
          widget.element.changePosition(
            widget.element.position + event.localDelta,
          );
        } else if (alignment == Alignment.topRight) {
          elementStartSize += -Offset(-event.localDelta.dx, event.localDelta.dy);
          var l = Offset(0, event.localDelta.dy);
          widget.element.changePosition(
            widget.element.position + l,
          );
        } else if (alignment == Alignment.bottomLeft) {
          elementStartSize += -Offset(event.localDelta.dx, -event.localDelta.dy);
          var l = Offset(event.localDelta.dx, 0);
          widget.element.changePosition(
            widget.element.position + l,
          );
        } else { elementStartSize += event.localDelta; }
        widget.element.changeSize(elementStartSize);
      },
      onPointerUp: (event) {
        widget.dashboard.setElementResizable(widget.element, false);
      },
      child: Align(
        alignment: alignment,
        child: InkWell( 
          mouseCursor: SystemMouseCursors.resizeColumn,
          child: HandlerWidget(
            width: 15,
            height: 15,
            borderColor: Colors.transparent,
            backgroundColor: widget.handlerColor,
            isRadius: false,
          ),
        ),
      ),
    );
  }
}
